from mayan.apps.dependencies.classes import (
    JavaScriptDependency, PythonDependency
)

JavaScriptDependency(
    module=__name__, name='signature_pad', replace_list=[
        {
            'filename_pattern': '*.umd.js',
            'content_patterns': [
                {
                    'search': '//# sourceMappingURL=signature_pad.umd.js.map',
                    'replace': '',
                }
            ]
        }
    ], version_string='=4.0.4'
)
PythonDependency(
    module=__name__, name='CairoSVG', version_string='==2.7.1'
)



from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('duplicates', '0008_auto_20201130_0847')
    ]

    operations = [
        migrations.RemoveField(
            model_name='duplicatebackendentry',
            name='datetime_added'
        ),
        migrations.AlterField(
            model_name='storedduplicatebackend',
            name='backend_data',
            field=models.TextField(
                blank=True, help_text='JSON encoded data for the backend '
                'class.', verbose_name='Backend data'
            )
        )
    ]



from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL)
    ]

    operations = [
        migrations.CreateModel(
            name='AutoAdminSingleton',
            fields=[
                (
                    'id', models.AutoField(
                        verbose_name='ID', serialize=False,
                        auto_created=True, primary_key=True
                    )
                ),
                (
                    'password', models.CharField(
                        max_length=128, null=True, verbose_name='Password',
                        blank=True
                    )
                ),
                (
                    'password_hash', models.CharField(
                        max_length=128, null=True,
                        verbose_name='Password hash', blank=True
                    )
                ),
                (
                    'account', models.ForeignKey(
                        verbose_name='Account', blank=True,
                        to=settings.AUTH_USER_MODEL, null=True,
                        on_delete=models.CASCADE
                    )
                )
            ],
            options={
                'verbose_name': 'Autoadmin properties',
                'verbose_name_plural': 'Autoadmin properties'
            },
            bases=(models.Model,)
        )
    ]



from mayan.apps.documents.permissions import permission_document_type_edit
from mayan.apps.documents.tests.base import GenericDocumentViewTestCase
from mayan.apps.testing.tests.base import GenericViewTestCase

from ..events import (
    event_workflow_instance_created, event_workflow_template_created,
    event_workflow_template_edited
)
from ..models.workflow_models import Workflow
from ..permissions import (
    permission_workflow_template_create,
    permission_workflow_template_delete, permission_workflow_template_edit,
    permission_workflow_template_view, permission_workflow_tools
)

from .literals import TEST_WORKFLOW_TEMPLATE_LABEL
from .mixins.workflow_instance_mixins import DocumentWorkflowTemplateViewTestMixin
from .mixins.workflow_template_mixins import (
    DocumentTypeAddRemoveWorkflowTemplateViewTestMixin,
    WorkflowTemplateDocumentTypeViewTestMixin, WorkflowTemplateViewTestMixin
)


class DocumentTypeAddRemoveWorkflowTemplateViewTestCase(
    DocumentTypeAddRemoveWorkflowTemplateViewTestMixin,
    GenericDocumentViewTestCase
):
    auto_create_test_workflow_template = True
    auto_upload_test_document = False

    def test_document_type_workflow_template_add_remove_get_view_no_permission(self):
        self._test_document_type.workflows.add(
            self._test_workflow_template
        )

        self._clear_events()

        response = self._request_test_document_type_workflow_template_add_remove_get_view()
        self.assertNotContains(
            response=response, text=str(self._test_document_type),
            status_code=404
        )
        self.assertNotContains(
            response=response, text=str(self._test_workflow_template),
            status_code=404
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_type_workflow_template_add_remove_get_view_with_document_type_access(self):
        self._test_document_type.workflows.add(
            self._test_workflow_template
        )

        self.grant_access(
            obj=self._test_document_type,
            permission=permission_document_type_edit
        )

        self._clear_events()

        response = self._request_test_document_type_workflow_template_add_remove_get_view()
        self.assertContains(
            response=response, text=str(self._test_document_type),
            status_code=200
        )
        self.assertNotContains(
            response=response, text=str(self._test_workflow_template),
            status_code=200
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_type_workflow_template_add_remove_get_view_with_workflow_template_access(self):
        self._test_document_type.workflows.add(
            self._test_workflow_template
        )

        self.grant_access(
            obj=self._test_workflow_template,
            permission=permission_workflow_template_edit
        )

        self._clear_events()

        response = self._request_test_document_type_workflow_template_add_remove_get_view()
        self.assertNotContains(
            response=response, text=str(self._test_document_type),
            status_code=404
        )
        self.assertNotContains(
            response=response, text=str(self._test_workflow_template),
            status_code=404
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_type_workflow_template_add_remove_get_view_with_full_access(self):
        self._test_document_type.workflows.add(
            self._test_workflow_template
        )

        self.grant_access(
            obj=self._test_document_type,
            permission=permission_document_type_edit
        )
        self.grant_access(
            obj=self._test_workflow_template,
            permission=permission_workflow_template_edit
        )

        self._clear_events()

        response = self._request_test_document_type_workflow_template_add_remove_get_view()
        self.assertContains(
            response=response, text=str(self._test_document_type),
            status_code=200
        )
        self.assertContains(
            response=response, text=str(self._test_workflow_template),
            status_code=200
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_type_workflow_template_add_view_no_permission(self):
        self._clear_events()

        response = self._request_test_document_type_workflow_template_add_view()
        self.assertEqual(response.status_code, 404)

        self.assertTrue(
            self._test_workflow_template not in self._test_document_type.workflows.all()
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_type_workflow_template_add_view_with_document_type_access(self):
        self.grant_access(
            obj=self._test_document_type,
            permission=permission_document_type_edit
        )

        self._clear_events()

        response = self._request_test_document_type_workflow_template_add_view()
        self.assertEqual(response.status_code, 200)

        self.assertTrue(
            self._test_workflow_template not in self._test_document_type.workflows.all()
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_type_workflow_template_add_view_with_workflow_template_access(self):
        self.grant_access(
            obj=self._test_workflow_template,
            permission=permission_workflow_template_edit
        )

        self._clear_events()

        response = self._request_test_document_type_workflow_template_add_view()
        self.assertEqual(response.status_code, 404)

        self.assertTrue(
            self._test_workflow_template not in self._test_document_type.workflows.all()
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_type_workflow_template_add_view_with_full_access(self):
        self.grant_access(
            obj=self._test_document_type,
            permission=permission_document_type_edit
        )
        self.grant_access(
            obj=self._test_workflow_template,
            permission=permission_workflow_template_edit
        )

        self._clear_events()

        response = self._request_test_document_type_workflow_template_add_view()
        self.assertEqual(response.status_code, 302)

        self.assertTrue(
            self._test_workflow_template in self._test_document_type.workflows.all()
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, self._test_document_type)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_workflow_template)
        self.assertEqual(events[0].verb, event_workflow_template_edited.id)

    def test_document_type_workflow_template_remove_view_no_permission(self):
        self._test_document_type.workflows.add(
            self._test_workflow_template
        )

        self._clear_events()

        response = self._request_test_document_type_workflow_template_remove_view()
        self.assertEqual(response.status_code, 404)

        self.assertTrue(
            self._test_workflow_template in self._test_document_type.workflows.all()
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_type_workflow_template_remove_view_with_document_type_access(self):
        self._test_document_type.workflows.add(
            self._test_workflow_template
        )

        self.grant_access(
            obj=self._test_document_type,
            permission=permission_document_type_edit
        )

        self._clear_events()

        response = self._request_test_document_type_workflow_template_remove_view()
        self.assertEqual(response.status_code, 200)

        self.assertTrue(
            self._test_workflow_template in self._test_document_type.workflows.all()
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_type_workflow_template_remove_view_with_workflow_template_access(self):
        self._test_document_type.workflows.add(
            self._test_workflow_template
        )

        self.grant_access(
            obj=self._test_workflow_template,
            permission=permission_workflow_template_edit
        )

        self._clear_events()

        response = self._request_test_document_type_workflow_template_remove_view()
        self.assertEqual(response.status_code, 404)

        self.assertTrue(
            self._test_workflow_template in self._test_document_type.workflows.all()
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_type_workflow_template_remove_view_with_full_access(self):
        self._test_document_type.workflows.add(
            self._test_workflow_template
        )

        self.grant_access(
            obj=self._test_document_type,
            permission=permission_document_type_edit
        )
        self.grant_access(
            obj=self._test_workflow_template,
            permission=permission_workflow_template_edit
        )

        self._clear_events()

        response = self._request_test_document_type_workflow_template_remove_view()
        self.assertEqual(response.status_code, 302)

        self.assertTrue(
            self._test_workflow_template not in self._test_document_type.workflows.all()
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, self._test_document_type)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_workflow_template)
        self.assertEqual(events[0].verb, event_workflow_template_edited.id)


class DocumentWorkflowTemplateViewTestCase(
    DocumentWorkflowTemplateViewTestMixin, GenericDocumentViewTestCase
):
    auto_upload_test_document = False

    def setUp(self):
        super().setUp()
        self._create_test_workflow_template(add_test_document_type=True)
        self._create_test_workflow_template_state()
        self._create_test_workflow_template_state()
        self._create_test_workflow_template_transition()
        self._test_workflow_template.auto_launch = False
        self._test_workflow_template.save()

    def test_document_single_workflow_launch_view_no_permission(self):
        self._create_test_document_stub()

        workflow_instance_count = self._test_document.workflows.count()

        self._clear_events()

        response = self._request_test_document_single_workflow_template_launch_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(
            self._test_document.workflows.count(), workflow_instance_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_single_workflow_launch_view_with_document_access(self):
        self._create_test_document_stub()

        self.grant_access(
            obj=self._test_document, permission=permission_workflow_tools
        )

        workflow_instance_count = self._test_document.workflows.count()

        self._clear_events()

        response = self._request_test_document_single_workflow_template_launch_view()
        self.assertEqual(response.status_code, 200)

        self.assertEqual(
            self._test_document.workflows.count(), workflow_instance_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_single_workflow_launch_view_with_workflow_access(self):
        self._create_test_document_stub()

        self.grant_access(
            obj=self._test_workflow_template,
            permission=permission_workflow_tools
        )

        workflow_instance_count = self._test_document.workflows.count()

        self._clear_events()

        response = self._request_test_document_single_workflow_template_launch_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(
            self._test_document.workflows.count(), workflow_instance_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_single_workflow_launch_view_with_full_access(self):
        self._create_test_document_stub()

        self.grant_access(
            obj=self._test_document, permission=permission_workflow_tools
        )
        self.grant_access(
            obj=self._test_workflow_template,
            permission=permission_workflow_tools
        )

        workflow_instance_count = self._test_document.workflows.count()

        self._clear_events()

        response = self._request_test_document_single_workflow_template_launch_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            self._test_document.workflows.count(), workflow_instance_count + 1
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, self._test_document)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(
            events[0].target, self._test_document.workflows.first()
        )
        self.assertEqual(events[0].verb, event_workflow_instance_created.id)

    def test_trashed_document_single_workflow_launch_view_with_full_access(self):
        self._create_test_document_stub()

        self.grant_access(
            obj=self._test_document, permission=permission_workflow_tools
        )
        self.grant_access(
            obj=self._test_workflow_template,
            permission=permission_workflow_tools
        )

        workflow_instance_count = self._test_document.workflows.count()

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_document_single_workflow_template_launch_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(
            self._test_document.workflows.count(), workflow_instance_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)


class WorkflowTemplateDocumentTypeViewTestCase(
    WorkflowTemplateDocumentTypeViewTestMixin, GenericDocumentViewTestCase
):
    auto_create_test_workflow_template = True
    auto_upload_test_document = False

    def test_workflow_template_document_type_add_remove_get_view_no_permission(self):
        self._test_workflow_template.document_types.add(
            self._test_document_type
        )

        test_workflow_template_document_type_count = self._test_workflow_template.document_types.count()

        self._clear_events()

        response = self._request_test_workflow_template_document_type_add_remove_get_view()
        self.assertNotContains(
            response=response, text=str(self._test_document_type),
            status_code=404
        )
        self.assertNotContains(
            response=response, text=str(self._test_workflow_template),
            status_code=404
        )

        self.assertEqual(
            self._test_workflow_template.document_types.count(),
            test_workflow_template_document_type_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_workflow_template_document_type_add_remove_get_view_with_document_type_access(self):
        self._test_workflow_template.document_types.add(
            self._test_document_type
        )

        test_workflow_template_document_type_count = self._test_workflow_template.document_types.count()

        self.grant_access(
            obj=self._test_document_type,
            permission=permission_document_type_edit
        )

        self._clear_events()

        response = self._request_test_workflow_template_document_type_add_remove_get_view()
        self.assertNotContains(
            response=response, text=str(self._test_document_type),
            status_code=404
        )
        self.assertNotContains(
            response=response, text=str(self._test_workflow_template),
            status_code=404
        )

        self.assertEqual(
            self._test_workflow_template.document_types.count(),
            test_workflow_template_document_type_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_workflow_template_document_type_add_remove_get_view_with_workflow_template_access(self):
        self._test_workflow_template.document_types.add(
            self._test_document_type
        )

        test_workflow_template_document_type_count = self._test_workflow_template.document_types.count()

        self.grant_access(
            obj=self._test_workflow_template,
            permission=permission_workflow_template_edit
        )

        self._clear_events()

        response = self._request_test_workflow_template_document_type_add_remove_get_view()
        self.assertNotContains(
            response=response, text=str(self._test_document_type),
            status_code=200
        )
        self.assertContains(
            response=response, text=str(self._test_workflow_template),
            status_code=200
        )

        self.assertEqual(
            self._test_workflow_template.document_types.count(),
            test_workflow_template_document_type_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_workflow_template_document_type_add_remove_get_view_with_full_access(self):
        self._test_workflow_template.document_types.add(
            self._test_document_type
        )

        test_workflow_template_document_type_count = self._test_workflow_template.document_types.count()

        self.grant_access(
            obj=self._test_document_type,
            permission=permission_document_type_edit
        )
        self.grant_access(
            obj=self._test_workflow_template,
            permission=permission_workflow_template_edit
        )

        self._clear_events()

        response = self._request_test_workflow_template_document_type_add_remove_get_view()
        self.assertContains(
            response=response, text=str(self._test_document_type),
            status_code=200
        )
        self.assertContains(
            response=response, text=str(self._test_workflow_template),
            status_code=200
        )

        self.assertEqual(
            self._test_workflow_template.document_types.count(),
            test_workflow_template_document_type_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_workflow_template_document_type_add_view_no_permission(self):
        test_workflow_template_document_type_count = self._test_workflow_template.document_types.count()

        self.grant_access(
            obj=self._test_document_type,
            permission=permission_document_type_edit
        )

        self._clear_events()

        response = self._request_test_workflow_template_document_type_add_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(
            self._test_workflow_template.document_types.count(),
            test_workflow_template_document_type_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_workflow_template_document_type_add_view_with_document_type_access(self):
        test_workflow_template_document_type_count = self._test_workflow_template.document_types.count()

        self.grant_access(
            obj=self._test_document_type,
            permission=permission_document_type_edit
        )

        self._clear_events()

        response = self._request_test_workflow_template_document_type_add_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(
            self._test_workflow_template.document_types.count(),
            test_workflow_template_document_type_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_workflow_template_document_type_add_view_with_workflow_template_access(self):
        test_workflow_template_document_type_count = self._test_workflow_template.document_types.count()

        self.grant_access(
            obj=self._test_workflow_template,
            permission=permission_workflow_template_edit
        )

        self._clear_events()

        response = self._request_test_workflow_template_document_type_add_view()
        self.assertEqual(response.status_code, 200)

        self.assertEqual(
            self._test_workflow_template.document_types.count(),
            test_workflow_template_document_type_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_workflow_template_document_type_add_view_with_full_access(self):
        test_workflow_template_document_type_count = self._test_workflow_template.document_types.count()

        self.grant_access(
            obj=self._test_document_type,
            permission=permission_document_type_edit
        )
        self.grant_access(
            obj=self._test_workflow_template,
            permission=permission_workflow_template_edit
        )

        self._clear_events()

        response = self._request_test_workflow_template_document_type_add_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            self._test_workflow_template.document_types.count(),
            test_workflow_template_document_type_count + 1
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, self._test_document_type)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_workflow_template)
        self.assertEqual(events[0].verb, event_workflow_template_edited.id)

    def test_workflow_template_document_type_remove_view_no_permission(self):
        self._test_workflow_template.document_types.add(
            self._test_document_type
        )

        test_workflow_template_document_type_count = self._test_workflow_template.document_types.count()

        self.grant_access(
            obj=self._test_document_type,
            permission=permission_document_type_edit
        )

        self._clear_events()

        response = self._request_test_workflow_template_document_type_remove_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(
            self._test_workflow_template.document_types.count(),
            test_workflow_template_document_type_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_workflow_template_document_type_remove_view_with_document_type_access(self):
        self._test_workflow_template.document_types.add(
            self._test_document_type
        )

        test_workflow_template_document_type_count = self._test_workflow_template.document_types.count()

        self.grant_access(
            obj=self._test_document_type,
            permission=permission_document_type_edit
        )

        self._clear_events()

        response = self._request_test_workflow_template_document_type_remove_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(
            self._test_workflow_template.document_types.count(),
            test_workflow_template_document_type_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_workflow_template_document_type_remove_view_with_workflow_template_access(self):
        self._test_workflow_template.document_types.add(
            self._test_document_type
        )

        test_workflow_template_document_type_count = self._test_workflow_template.document_types.count()

        self.grant_access(
            obj=self._test_workflow_template,
            permission=permission_workflow_template_edit
        )

        self._clear_events()

        response = self._request_test_workflow_template_document_type_remove_view()
        self.assertEqual(response.status_code, 200)

        self.assertEqual(
            self._test_workflow_template.document_types.count(),
            test_workflow_template_document_type_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_workflow_template_document_type_remove_view_with_full_access(self):
        self._test_workflow_template.document_types.add(
            self._test_document_type
        )

        test_workflow_template_document_type_count = self._test_workflow_template.document_types.count()

        self.grant_access(
            obj=self._test_document_type,
            permission=permission_document_type_edit
        )
        self.grant_access(
            obj=self._test_workflow_template,
            permission=permission_workflow_template_edit
        )

        self._clear_events()

        response = self._request_test_workflow_template_document_type_remove_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            self._test_workflow_template.document_types.count(),
            test_workflow_template_document_type_count - 1
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, self._test_document_type)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_workflow_template)
        self.assertEqual(events[0].verb, event_workflow_template_edited.id)


class WorkflowTemplateViewTestCase(
    WorkflowTemplateViewTestMixin, GenericViewTestCase
):
    def test_workflow_template_create_view_no_permission(self):
        self._clear_events()

        response = self._request_test_workflow_template_create_view()
        self.assertEqual(response.status_code, 403)

        self.assertEqual(
            Workflow.objects.count(), 0
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_workflow_template_create_view_with_permission(self):
        self.grant_permission(permission=permission_workflow_template_create)

        self._clear_events()

        response = self._request_test_workflow_template_create_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            Workflow.objects.count(), 1
        )
        self.assertEqual(
            Workflow.objects.all()[0].label, TEST_WORKFLOW_TEMPLATE_LABEL
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_workflow_template)
        self.assertEqual(events[0].verb, event_workflow_template_created.id)

    def test_workflow_template_delete_view_no_permission(self):
        self._create_test_workflow_template()

        self._clear_events()

        response = self._request_test_workflow_template_delete_view()
        self.assertEqual(response.status_code, 404)

        self.assertTrue(
            self._test_workflow_template in Workflow.objects.all()
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_workflow_template_delete_view_with_access(self):
        self._create_test_workflow_template()

        self.grant_access(
            obj=self._test_workflow_template,
            permission=permission_workflow_template_delete
        )

        self._clear_events()

        response = self._request_test_workflow_template_delete_view()
        self.assertEqual(response.status_code, 302)

        self.assertFalse(
            self._test_workflow_template in Workflow.objects.all()
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_workflow_template_edit_view_no_permission(self):
        self._create_test_workflow_template()

        test_workflow_template_label = self._test_workflow_template.label

        self._clear_events()

        response = self._request_test_workflow_template_edit_view()
        self.assertEqual(response.status_code, 404)

        self._test_workflow_template.refresh_from_db()
        self.assertEqual(
            self._test_workflow_template.label, test_workflow_template_label
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_workflow_template_edit_view_with_access(self):
        self._create_test_workflow_template()

        test_workflow_template_label = self._test_workflow_template.label

        self.grant_access(
            obj=self._test_workflow_template,
            permission=permission_workflow_template_edit
        )

        self._clear_events()

        response = self._request_test_workflow_template_edit_view()
        self.assertEqual(response.status_code, 302)

        self._test_workflow_template.refresh_from_db()
        self.assertNotEqual(
            self._test_workflow_template.label, test_workflow_template_label
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_workflow_template)
        self.assertEqual(events[0].verb, event_workflow_template_edited.id)

    def test_workflow_template_list_view_no_permission(self):
        self._create_test_workflow_template()

        self._clear_events()

        response = self._request_test_workflow_template_list_view()

        self.assertNotContains(
            response=response, text=self._test_workflow_template.label,
            status_code=200
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_workflow_template_list_view_with_access(self):
        self._create_test_workflow_template()

        self.grant_access(
            obj=self._test_workflow_template,
            permission=permission_workflow_template_view
        )

        self._clear_events()

        response = self._request_test_workflow_template_list_view()
        self.assertContains(
            response=response, text=self._test_workflow_template.label,
            status_code=200
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_workflow_template_preview_view_no_permission(self):
        self._create_test_workflow_template()

        self._clear_events()

        response = self._request_test_workflow_template_preview_view()
        self.assertEqual(response.status_code, 404)

        self.assertTrue(
            self._test_workflow_template in Workflow.objects.all()
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_workflow_template_preview_view_with_access(self):
        self._create_test_workflow_template()

        self.grant_access(
            obj=self._test_workflow_template,
            permission=permission_workflow_template_view
        )

        self._clear_events()

        response = self._request_test_workflow_template_preview_view()
        self.assertEqual(response.status_code, 200)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)



# -*- coding: utf-8 -*-
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('common', '0006_auto_20160313_0313')
    ]

    operations = [
        migrations.AlterField(
            model_name='userlocaleprofile',
            name='timezone',
            field=models.CharField(
                choices=[
                    (b'Africa/Abidjan', b'Africa/Abidjan'),
                    (b'Africa/Accra', b'Africa/Accra'),
                    (b'Africa/Addis_Ababa', b'Africa/Addis_Ababa'),
                    (b'Africa/Algiers', b'Africa/Algiers'),
                    (b'Africa/Asmara', b'Africa/Asmara'),
                    (b'Africa/Bamako', b'Africa/Bamako'),
                    (b'Africa/Bangui', b'Africa/Bangui'),
                    (b'Africa/Banjul', b'Africa/Banjul'),
                    (b'Africa/Bissau', b'Africa/Bissau'),
                    (b'Africa/Blantyre', b'Africa/Blantyre'),
                    (b'Africa/Brazzaville', b'Africa/Brazzaville'),
                    (b'Africa/Bujumbura', b'Africa/Bujumbura'),
                    (b'Africa/Cairo', b'Africa/Cairo'),
                    (b'Africa/Casablanca', b'Africa/Casablanca'),
                    (b'Africa/Ceuta', b'Africa/Ceuta'),
                    (b'Africa/Conakry', b'Africa/Conakry'),
                    (b'Africa/Dakar', b'Africa/Dakar'),
                    (b'Africa/Dar_es_Salaam', b'Africa/Dar_es_Salaam'),
                    (b'Africa/Djibouti', b'Africa/Djibouti'),
                    (b'Africa/Douala', b'Africa/Douala'),
                    (b'Africa/El_Aaiun', b'Africa/El_Aaiun'),
                    (b'Africa/Freetown', b'Africa/Freetown'),
                    (b'Africa/Gaborone', b'Africa/Gaborone'),
                    (b'Africa/Harare', b'Africa/Harare'),
                    (b'Africa/Johannesburg', b'Africa/Johannesburg'),
                    (b'Africa/Juba', b'Africa/Juba'),
                    (b'Africa/Kampala', b'Africa/Kampala'),
                    (b'Africa/Khartoum', b'Africa/Khartoum'),
                    (b'Africa/Kigali', b'Africa/Kigali'),
                    (b'Africa/Kinshasa', b'Africa/Kinshasa'),
                    (b'Africa/Lagos', b'Africa/Lagos'),
                    (b'Africa/Libreville', b'Africa/Libreville'),
                    (b'Africa/Lome', b'Africa/Lome'),
                    (b'Africa/Luanda', b'Africa/Luanda'),
                    (b'Africa/Lubumbashi', b'Africa/Lubumbashi'),
                    (b'Africa/Lusaka', b'Africa/Lusaka'),
                    (b'Africa/Malabo', b'Africa/Malabo'),
                    (b'Africa/Maputo', b'Africa/Maputo'),
                    (b'Africa/Maseru', b'Africa/Maseru'),
                    (b'Africa/Mbabane', b'Africa/Mbabane'),
                    (b'Africa/Mogadishu', b'Africa/Mogadishu'),
                    (b'Africa/Monrovia', b'Africa/Monrovia'),
                    (b'Africa/Nairobi', b'Africa/Nairobi'),
                    (b'Africa/Ndjamena', b'Africa/Ndjamena'),
                    (b'Africa/Niamey', b'Africa/Niamey'),
                    (b'Africa/Nouakchott', b'Africa/Nouakchott'),
                    (b'Africa/Ouagadougou', b'Africa/Ouagadougou'),
                    (b'Africa/Porto-Novo', b'Africa/Porto-Novo'),
                    (b'Africa/Sao_Tome', b'Africa/Sao_Tome'),
                    (b'Africa/Tripoli', b'Africa/Tripoli'),
                    (b'Africa/Tunis', b'Africa/Tunis'),
                    (b'Africa/Windhoek', b'Africa/Windhoek'),
                    (b'America/Adak', b'America/Adak'),
                    (b'America/Anchorage', b'America/Anchorage'),
                    (b'America/Anguilla', b'America/Anguilla'),
                    (b'America/Antigua', b'America/Antigua'),
                    (b'America/Araguaina', b'America/Araguaina'),
                    (
                        b'America/Argentina/Buenos_Aires',
                        b'America/Argentina/Buenos_Aires'
                    ),
                    (
                        b'America/Argentina/Catamarca',
                        b'America/Argentina/Catamarca'
                    ),
                    (
                        b'America/Argentina/Cordoba',
                        b'America/Argentina/Cordoba'
                    ),
                    (b'America/Argentina/Jujuy', b'America/Argentina/Jujuy'),
                    (
                        b'America/Argentina/La_Rioja',
                        b'America/Argentina/La_Rioja'
                    ),
                    (
                        b'America/Argentina/Mendoza',
                        b'America/Argentina/Mendoza'
                    ),
                    (
                        b'America/Argentina/Rio_Gallegos',
                        b'America/Argentina/Rio_Gallegos'
                    ),
                    (b'America/Argentina/Salta', b'America/Argentina/Salta'),
                    (
                        b'America/Argentina/San_Juan',
                        b'America/Argentina/San_Juan'
                    ),
                    (
                        b'America/Argentina/San_Luis',
                        b'America/Argentina/San_Luis'
                    ),
                    (
                        b'America/Argentina/Tucuman',
                        b'America/Argentina/Tucuman'
                    ),
                    (
                        b'America/Argentina/Ushuaia',
                        b'America/Argentina/Ushuaia'
                    ),
                    (b'America/Aruba', b'America/Aruba'),
                    (b'America/Asuncion', b'America/Asuncion'),
                    (b'America/Atikokan', b'America/Atikokan'),
                    (b'America/Bahia', b'America/Bahia'),
                    (b'America/Bahia_Banderas', b'America/Bahia_Banderas'),
                    (b'America/Barbados', b'America/Barbados'),
                    (b'America/Belem', b'America/Belem'),
                    (b'America/Belize', b'America/Belize'),
                    (b'America/Blanc-Sablon', b'America/Blanc-Sablon'),
                    (b'America/Boa_Vista', b'America/Boa_Vista'),
                    (b'America/Bogota', b'America/Bogota'),
                    (b'America/Boise', b'America/Boise'),
                    (b'America/Cambridge_Bay', b'America/Cambridge_Bay'),
                    (b'America/Campo_Grande', b'America/Campo_Grande'),
                    (b'America/Cancun', b'America/Cancun'),
                    (b'America/Caracas', b'America/Caracas'),
                    (b'America/Cayenne', b'America/Cayenne'),
                    (b'America/Cayman', b'America/Cayman'),
                    (b'America/Chicago', b'America/Chicago'),
                    (b'America/Chihuahua', b'America/Chihuahua'),
                    (b'America/Costa_Rica', b'America/Costa_Rica'),
                    (b'America/Creston', b'America/Creston'),
                    (b'America/Cuiaba', b'America/Cuiaba'),
                    (b'America/Curacao', b'America/Curacao'),
                    (b'America/Danmarkshavn', b'America/Danmarkshavn'),
                    (b'America/Dawson', b'America/Dawson'),
                    (b'America/Dawson_Creek', b'America/Dawson_Creek'),
                    (b'America/Denver', b'America/Denver'),
                    (b'America/Detroit', b'America/Detroit'),
                    (b'America/Dominica', b'America/Dominica'),
                    (b'America/Edmonton', b'America/Edmonton'),
                    (b'America/Eirunepe', b'America/Eirunepe'),
                    (b'America/El_Salvador', b'America/El_Salvador'),
                    (b'America/Fort_Nelson', b'America/Fort_Nelson'),
                    (b'America/Fortaleza', b'America/Fortaleza'),
                    (b'America/Glace_Bay', b'America/Glace_Bay'),
                    (b'America/Godthab', b'America/Godthab'),
                    (b'America/Goose_Bay', b'America/Goose_Bay'),
                    (b'America/Grand_Turk', b'America/Grand_Turk'),
                    (b'America/Grenada', b'America/Grenada'),
                    (b'America/Guadeloupe', b'America/Guadeloupe'),
                    (b'America/Guatemala', b'America/Guatemala'),
                    (b'America/Guayaquil', b'America/Guayaquil'),
                    (b'America/Guyana', b'America/Guyana'),
                    (b'America/Halifax', b'America/Halifax'),
                    (b'America/Havana', b'America/Havana'),
                    (b'America/Hermosillo', b'America/Hermosillo'),
                    (
                        b'America/Indiana/Indianapolis',
                        b'America/Indiana/Indianapolis'
                    ),
                    (b'America/Indiana/Knox', b'America/Indiana/Knox'),
                    (b'America/Indiana/Marengo', b'America/Indiana/Marengo'),
                    (
                        b'America/Indiana/Petersburg',
                        b'America/Indiana/Petersburg'
                    ),
                    (
                        b'America/Indiana/Tell_City',
                        b'America/Indiana/Tell_City'
                    ),
                    (b'America/Indiana/Vevay', b'America/Indiana/Vevay'),
                    (
                        b'America/Indiana/Vincennes',
                        b'America/Indiana/Vincennes'
                    ),
                    (b'America/Indiana/Winamac', b'America/Indiana/Winamac'),
                    (b'America/Inuvik', b'America/Inuvik'),
                    (b'America/Iqaluit', b'America/Iqaluit'),
                    (b'America/Jamaica', b'America/Jamaica'),
                    (b'America/Juneau', b'America/Juneau'),
                    (
                        b'America/Kentucky/Louisville',
                        b'America/Kentucky/Louisville'
                    ),
                    (
                        b'America/Kentucky/Monticello',
                        b'America/Kentucky/Monticello'
                    ),
                    (b'America/Kralendijk', b'America/Kralendijk'),
                    (b'America/La_Paz', b'America/La_Paz'),
                    (b'America/Lima', b'America/Lima'),
                    (b'America/Los_Angeles', b'America/Los_Angeles'),
                    (b'America/Lower_Princes', b'America/Lower_Princes'),
                    (b'America/Maceio', b'America/Maceio'),
                    (b'America/Managua', b'America/Managua'),
                    (b'America/Manaus', b'America/Manaus'),
                    (b'America/Marigot', b'America/Marigot'),
                    (b'America/Martinique', b'America/Martinique'),
                    (b'America/Matamoros', b'America/Matamoros'),
                    (b'America/Mazatlan', b'America/Mazatlan'),
                    (b'America/Menominee', b'America/Menominee'),
                    (b'America/Merida', b'America/Merida'),
                    (b'America/Metlakatla', b'America/Metlakatla'),
                    (b'America/Mexico_City', b'America/Mexico_City'),
                    (b'America/Miquelon', b'America/Miquelon'),
                    (b'America/Moncton', b'America/Moncton'),
                    (b'America/Monterrey', b'America/Monterrey'),
                    (b'America/Montevideo', b'America/Montevideo'),
                    (b'America/Montserrat', b'America/Montserrat'),
                    (b'America/Nassau', b'America/Nassau'),
                    (b'America/New_York', b'America/New_York'),
                    (b'America/Nipigon', b'America/Nipigon'),
                    (b'America/Nome', b'America/Nome'),
                    (b'America/Noronha', b'America/Noronha'),
                    (
                        b'America/North_Dakota/Beulah',
                        b'America/North_Dakota/Beulah'
                    ),
                    (
                        b'America/North_Dakota/Center',
                        b'America/North_Dakota/Center'
                    ),
                    (
                        b'America/North_Dakota/New_Salem',
                        b'America/North_Dakota/New_Salem'
                    ),
                    (b'America/Ojinaga', b'America/Ojinaga'),
                    (b'America/Panama', b'America/Panama'),
                    (b'America/Pangnirtung', b'America/Pangnirtung'),
                    (b'America/Paramaribo', b'America/Paramaribo'),
                    (b'America/Phoenix', b'America/Phoenix'),
                    (b'America/Port-au-Prince', b'America/Port-au-Prince'),
                    (b'America/Port_of_Spain', b'America/Port_of_Spain'),
                    (b'America/Porto_Velho', b'America/Porto_Velho'),
                    (b'America/Puerto_Rico', b'America/Puerto_Rico'),
                    (b'America/Rainy_River', b'America/Rainy_River'),
                    (b'America/Rankin_Inlet', b'America/Rankin_Inlet'),
                    (b'America/Recife', b'America/Recife'),
                    (b'America/Regina', b'America/Regina'),
                    (b'America/Resolute', b'America/Resolute'),
                    (b'America/Rio_Branco', b'America/Rio_Branco'),
                    (b'America/Santarem', b'America/Santarem'),
                    (b'America/Santiago', b'America/Santiago'),
                    (b'America/Santo_Domingo', b'America/Santo_Domingo'),
                    (b'America/Sao_Paulo', b'America/Sao_Paulo'),
                    (b'America/Scoresbysund', b'America/Scoresbysund'),
                    (b'America/Sitka', b'America/Sitka'),
                    (b'America/St_Barthelemy', b'America/St_Barthelemy'),
                    (b'America/St_Johns', b'America/St_Johns'),
                    (b'America/St_Kitts', b'America/St_Kitts'),
                    (b'America/St_Lucia', b'America/St_Lucia'),
                    (b'America/St_Thomas', b'America/St_Thomas'),
                    (b'America/St_Vincent', b'America/St_Vincent'),
                    (b'America/Swift_Current', b'America/Swift_Current'),
                    (b'America/Tegucigalpa', b'America/Tegucigalpa'),
                    (b'America/Thule', b'America/Thule'),
                    (b'America/Thunder_Bay', b'America/Thunder_Bay'),
                    (b'America/Tijuana', b'America/Tijuana'),
                    (b'America/Toronto', b'America/Toronto'),
                    (b'America/Tortola', b'America/Tortola'),
                    (b'America/Vancouver', b'America/Vancouver'),
                    (b'America/Whitehorse', b'America/Whitehorse'),
                    (b'America/Winnipeg', b'America/Winnipeg'),
                    (b'America/Yakutat', b'America/Yakutat'),
                    (b'America/Yellowknife', b'America/Yellowknife'),
                    (b'Antarctica/Casey', b'Antarctica/Casey'),
                    (b'Antarctica/Davis', b'Antarctica/Davis'),
                    (
                        b'Antarctica/DumontDUrville',
                        b'Antarctica/DumontDUrville'
                    ),
                    (b'Antarctica/Macquarie', b'Antarctica/Macquarie'),
                    (b'Antarctica/Mawson', b'Antarctica/Mawson'),
                    (b'Antarctica/McMurdo', b'Antarctica/McMurdo'),
                    (b'Antarctica/Palmer', b'Antarctica/Palmer'),
                    (b'Antarctica/Rothera', b'Antarctica/Rothera'),
                    (b'Antarctica/Syowa', b'Antarctica/Syowa'),
                    (b'Antarctica/Troll', b'Antarctica/Troll'),
                    (b'Antarctica/Vostok', b'Antarctica/Vostok'),
                    (b'Arctic/Longyearbyen', b'Arctic/Longyearbyen'),
                    (b'Asia/Aden', b'Asia/Aden'),
                    (b'Asia/Almaty', b'Asia/Almaty'),
                    (b'Asia/Amman', b'Asia/Amman'),
                    (b'Asia/Anadyr', b'Asia/Anadyr'),
                    (b'Asia/Aqtau', b'Asia/Aqtau'),
                    (b'Asia/Aqtobe', b'Asia/Aqtobe'),
                    (b'Asia/Ashgabat', b'Asia/Ashgabat'),
                    (b'Asia/Baghdad', b'Asia/Baghdad'),
                    (b'Asia/Bahrain', b'Asia/Bahrain'),
                    (b'Asia/Baku', b'Asia/Baku'),
                    (b'Asia/Bangkok', b'Asia/Bangkok'),
                    (b'Asia/Barnaul', b'Asia/Barnaul'),
                    (b'Asia/Beirut', b'Asia/Beirut'),
                    (b'Asia/Bishkek', b'Asia/Bishkek'),
                    (b'Asia/Brunei', b'Asia/Brunei'),
                    (b'Asia/Chita', b'Asia/Chita'),
                    (b'Asia/Choibalsan', b'Asia/Choibalsan'),
                    (b'Asia/Colombo', b'Asia/Colombo'),
                    (b'Asia/Damascus', b'Asia/Damascus'),
                    (b'Asia/Dhaka', b'Asia/Dhaka'),
                    (b'Asia/Dili', b'Asia/Dili'),
                    (b'Asia/Dubai', b'Asia/Dubai'),
                    (b'Asia/Dushanbe', b'Asia/Dushanbe'),
                    (b'Asia/Gaza', b'Asia/Gaza'),
                    (b'Asia/Hebron', b'Asia/Hebron'),
                    (b'Asia/Ho_Chi_Minh', b'Asia/Ho_Chi_Minh'),
                    (b'Asia/Hong_Kong', b'Asia/Hong_Kong'),
                    (b'Asia/Hovd', b'Asia/Hovd'),
                    (b'Asia/Irkutsk', b'Asia/Irkutsk'),
                    (b'Asia/Jakarta', b'Asia/Jakarta'),
                    (b'Asia/Jayapura', b'Asia/Jayapura'),
                    (b'Asia/Jerusalem', b'Asia/Jerusalem'),
                    (b'Asia/Kabul', b'Asia/Kabul'),
                    (b'Asia/Kamchatka', b'Asia/Kamchatka'),
                    (b'Asia/Karachi', b'Asia/Karachi'),
                    (b'Asia/Kathmandu', b'Asia/Kathmandu'),
                    (b'Asia/Khandyga', b'Asia/Khandyga'),
                    (b'Asia/Kolkata', b'Asia/Kolkata'),
                    (b'Asia/Krasnoyarsk', b'Asia/Krasnoyarsk'),
                    (b'Asia/Kuala_Lumpur', b'Asia/Kuala_Lumpur'),
                    (b'Asia/Kuching', b'Asia/Kuching'),
                    (b'Asia/Kuwait', b'Asia/Kuwait'),
                    (b'Asia/Macau', b'Asia/Macau'),
                    (b'Asia/Magadan', b'Asia/Magadan'),
                    (b'Asia/Makassar', b'Asia/Makassar'),
                    (b'Asia/Manila', b'Asia/Manila'),
                    (b'Asia/Muscat', b'Asia/Muscat'),
                    (b'Asia/Nicosia', b'Asia/Nicosia'),
                    (b'Asia/Novokuznetsk', b'Asia/Novokuznetsk'),
                    (b'Asia/Novosibirsk', b'Asia/Novosibirsk'),
                    (b'Asia/Omsk', b'Asia/Omsk'),
                    (b'Asia/Oral', b'Asia/Oral'),
                    (b'Asia/Phnom_Penh', b'Asia/Phnom_Penh'),
                    (b'Asia/Pontianak', b'Asia/Pontianak'),
                    (b'Asia/Pyongyang', b'Asia/Pyongyang'),
                    (b'Asia/Qatar', b'Asia/Qatar'),
                    (b'Asia/Qyzylorda', b'Asia/Qyzylorda'),
                    (b'Asia/Riyadh', b'Asia/Riyadh'),
                    (b'Asia/Sakhalin', b'Asia/Sakhalin'),
                    (b'Asia/Samarkand', b'Asia/Samarkand'),
                    (b'Asia/Seoul', b'Asia/Seoul'),
                    (b'Asia/Shanghai', b'Asia/Shanghai'),
                    (b'Asia/Singapore', b'Asia/Singapore'),
                    (b'Asia/Srednekolymsk', b'Asia/Srednekolymsk'),
                    (b'Asia/Taipei', b'Asia/Taipei'),
                    (b'Asia/Tashkent', b'Asia/Tashkent'),
                    (b'Asia/Tbilisi', b'Asia/Tbilisi'),
                    (b'Asia/Tehran', b'Asia/Tehran'),
                    (b'Asia/Thimphu', b'Asia/Thimphu'),
                    (b'Asia/Tokyo', b'Asia/Tokyo'),
                    (b'Asia/Tomsk', b'Asia/Tomsk'),
                    (b'Asia/Ulaanbaatar', b'Asia/Ulaanbaatar'),
                    (b'Asia/Urumqi', b'Asia/Urumqi'),
                    (b'Asia/Ust-Nera', b'Asia/Ust-Nera'),
                    (b'Asia/Vientiane', b'Asia/Vientiane'),
                    (b'Asia/Vladivostok', b'Asia/Vladivostok'),
                    (b'Asia/Yakutsk', b'Asia/Yakutsk'),
                    (b'Asia/Yangon', b'Asia/Yangon'),
                    (b'Asia/Yekaterinburg', b'Asia/Yekaterinburg'),
                    (b'Asia/Yerevan', b'Asia/Yerevan'),
                    (b'Atlantic/Azores', b'Atlantic/Azores'),
                    (b'Atlantic/Bermuda', b'Atlantic/Bermuda'),
                    (b'Atlantic/Canary', b'Atlantic/Canary'),
                    (b'Atlantic/Cape_Verde', b'Atlantic/Cape_Verde'),
                    (b'Atlantic/Faroe', b'Atlantic/Faroe'),
                    (b'Atlantic/Madeira', b'Atlantic/Madeira'),
                    (b'Atlantic/Reykjavik', b'Atlantic/Reykjavik'),
                    (b'Atlantic/South_Georgia', b'Atlantic/South_Georgia'),
                    (b'Atlantic/St_Helena', b'Atlantic/St_Helena'),
                    (b'Atlantic/Stanley', b'Atlantic/Stanley'),
                    (b'Australia/Adelaide', b'Australia/Adelaide'),
                    (b'Australia/Brisbane', b'Australia/Brisbane'),
                    (b'Australia/Broken_Hill', b'Australia/Broken_Hill'),
                    (b'Australia/Currie', b'Australia/Currie'),
                    (b'Australia/Darwin', b'Australia/Darwin'),
                    (b'Australia/Eucla', b'Australia/Eucla'),
                    (b'Australia/Hobart', b'Australia/Hobart'),
                    (b'Australia/Lindeman', b'Australia/Lindeman'),
                    (b'Australia/Lord_Howe', b'Australia/Lord_Howe'),
                    (b'Australia/Melbourne', b'Australia/Melbourne'),
                    (b'Australia/Perth', b'Australia/Perth'),
                    (b'Australia/Sydney', b'Australia/Sydney'),
                    (b'Canada/Atlantic', b'Canada/Atlantic'),
                    (b'Canada/Central', b'Canada/Central'),
                    (b'Canada/Eastern', b'Canada/Eastern'),
                    (b'Canada/Mountain', b'Canada/Mountain'),
                    (b'Canada/Newfoundland', b'Canada/Newfoundland'),
                    (b'Canada/Pacific', b'Canada/Pacific'),
                    (b'Europe/Amsterdam', b'Europe/Amsterdam'),
                    (b'Europe/Andorra', b'Europe/Andorra'),
                    (b'Europe/Astrakhan', b'Europe/Astrakhan'),
                    (b'Europe/Athens', b'Europe/Athens'),
                    (b'Europe/Belgrade', b'Europe/Belgrade'),
                    (b'Europe/Berlin', b'Europe/Berlin'),
                    (b'Europe/Bratislava', b'Europe/Bratislava'),
                    (b'Europe/Brussels', b'Europe/Brussels'),
                    (b'Europe/Bucharest', b'Europe/Bucharest'),
                    (b'Europe/Budapest', b'Europe/Budapest'),
                    (b'Europe/Busingen', b'Europe/Busingen'),
                    (b'Europe/Chisinau', b'Europe/Chisinau'),
                    (b'Europe/Copenhagen', b'Europe/Copenhagen'),
                    (b'Europe/Dublin', b'Europe/Dublin'),
                    (b'Europe/Gibraltar', b'Europe/Gibraltar'),
                    (b'Europe/Guernsey', b'Europe/Guernsey'),
                    (b'Europe/Helsinki', b'Europe/Helsinki'),
                    (b'Europe/Isle_of_Man', b'Europe/Isle_of_Man'),
                    (b'Europe/Istanbul', b'Europe/Istanbul'),
                    (b'Europe/Jersey', b'Europe/Jersey'),
                    (b'Europe/Kaliningrad', b'Europe/Kaliningrad'),
                    (b'Europe/Kiev', b'Europe/Kiev'),
                    (b'Europe/Kirov', b'Europe/Kirov'),
                    (b'Europe/Lisbon', b'Europe/Lisbon'),
                    (b'Europe/Ljubljana', b'Europe/Ljubljana'),
                    (b'Europe/London', b'Europe/London'),
                    (b'Europe/Luxembourg', b'Europe/Luxembourg'),
                    (b'Europe/Madrid', b'Europe/Madrid'),
                    (b'Europe/Malta', b'Europe/Malta'),
                    (b'Europe/Mariehamn', b'Europe/Mariehamn'),
                    (b'Europe/Minsk', b'Europe/Minsk'),
                    (b'Europe/Monaco', b'Europe/Monaco'),
                    (b'Europe/Moscow', b'Europe/Moscow'),
                    (b'Europe/Oslo', b'Europe/Oslo'),
                    (b'Europe/Paris', b'Europe/Paris'),
                    (b'Europe/Podgorica', b'Europe/Podgorica'),
                    (b'Europe/Prague', b'Europe/Prague'),
                    (b'Europe/Riga', b'Europe/Riga'),
                    (b'Europe/Rome', b'Europe/Rome'),
                    (b'Europe/Samara', b'Europe/Samara'),
                    (b'Europe/San_Marino', b'Europe/San_Marino'),
                    (b'Europe/Sarajevo', b'Europe/Sarajevo'),
                    (b'Europe/Simferopol', b'Europe/Simferopol'),
                    (b'Europe/Skopje', b'Europe/Skopje'),
                    (b'Europe/Sofia', b'Europe/Sofia'),
                    (b'Europe/Stockholm', b'Europe/Stockholm'),
                    (b'Europe/Tallinn', b'Europe/Tallinn'),
                    (b'Europe/Tirane', b'Europe/Tirane'),
                    (b'Europe/Ulyanovsk', b'Europe/Ulyanovsk'),
                    (b'Europe/Uzhgorod', b'Europe/Uzhgorod'),
                    (b'Europe/Vaduz', b'Europe/Vaduz'),
                    (b'Europe/Vatican', b'Europe/Vatican'),
                    (b'Europe/Vienna', b'Europe/Vienna'),
                    (b'Europe/Vilnius', b'Europe/Vilnius'),
                    (b'Europe/Volgograd', b'Europe/Volgograd'),
                    (b'Europe/Warsaw', b'Europe/Warsaw'),
                    (b'Europe/Zagreb', b'Europe/Zagreb'),
                    (b'Europe/Zaporozhye', b'Europe/Zaporozhye'),
                    (b'Europe/Zurich', b'Europe/Zurich'),
                    (b'GMT', b'GMT'),
                    (b'Indian/Antananarivo', b'Indian/Antananarivo'),
                    (b'Indian/Chagos', b'Indian/Chagos'),
                    (b'Indian/Christmas', b'Indian/Christmas'),
                    (b'Indian/Cocos', b'Indian/Cocos'),
                    (b'Indian/Comoro', b'Indian/Comoro'),
                    (b'Indian/Kerguelen', b'Indian/Kerguelen'),
                    (b'Indian/Mahe', b'Indian/Mahe'),
                    (b'Indian/Maldives', b'Indian/Maldives'),
                    (b'Indian/Mauritius', b'Indian/Mauritius'),
                    (b'Indian/Mayotte', b'Indian/Mayotte'),
                    (b'Indian/Reunion', b'Indian/Reunion'),
                    (b'Pacific/Apia', b'Pacific/Apia'),
                    (b'Pacific/Auckland', b'Pacific/Auckland'),
                    (b'Pacific/Bougainville', b'Pacific/Bougainville'),
                    (b'Pacific/Chatham', b'Pacific/Chatham'),
                    (b'Pacific/Chuuk', b'Pacific/Chuuk'),
                    (b'Pacific/Easter', b'Pacific/Easter'),
                    (b'Pacific/Efate', b'Pacific/Efate'),
                    (b'Pacific/Enderbury', b'Pacific/Enderbury'),
                    (b'Pacific/Fakaofo', b'Pacific/Fakaofo'),
                    (b'Pacific/Fiji', b'Pacific/Fiji'),
                    (b'Pacific/Funafuti', b'Pacific/Funafuti'),
                    (b'Pacific/Galapagos', b'Pacific/Galapagos'),
                    (b'Pacific/Gambier', b'Pacific/Gambier'),
                    (b'Pacific/Guadalcanal', b'Pacific/Guadalcanal'),
                    (b'Pacific/Guam', b'Pacific/Guam'),
                    (b'Pacific/Honolulu', b'Pacific/Honolulu'),
                    (b'Pacific/Johnston', b'Pacific/Johnston'),
                    (b'Pacific/Kiritimati', b'Pacific/Kiritimati'),
                    (b'Pacific/Kosrae', b'Pacific/Kosrae'),
                    (b'Pacific/Kwajalein', b'Pacific/Kwajalein'),
                    (b'Pacific/Majuro', b'Pacific/Majuro'),
                    (b'Pacific/Marquesas', b'Pacific/Marquesas'),
                    (b'Pacific/Midway', b'Pacific/Midway'),
                    (b'Pacific/Nauru', b'Pacific/Nauru'),
                    (b'Pacific/Niue', b'Pacific/Niue'),
                    (b'Pacific/Norfolk', b'Pacific/Norfolk'),
                    (b'Pacific/Noumea', b'Pacific/Noumea'),
                    (b'Pacific/Pago_Pago', b'Pacific/Pago_Pago'),
                    (b'Pacific/Palau', b'Pacific/Palau'),
                    (b'Pacific/Pitcairn', b'Pacific/Pitcairn'),
                    (b'Pacific/Pohnpei', b'Pacific/Pohnpei'),
                    (b'Pacific/Port_Moresby', b'Pacific/Port_Moresby'),
                    (b'Pacific/Rarotonga', b'Pacific/Rarotonga'),
                    (b'Pacific/Saipan', b'Pacific/Saipan'),
                    (b'Pacific/Tahiti', b'Pacific/Tahiti'),
                    (b'Pacific/Tarawa', b'Pacific/Tarawa'),
                    (b'Pacific/Tongatapu', b'Pacific/Tongatapu'),
                    (b'Pacific/Wake', b'Pacific/Wake'),
                    (b'Pacific/Wallis', b'Pacific/Wallis'),
                    (b'US/Alaska', b'US/Alaska'),
                    (b'US/Arizona', b'US/Arizona'),
                    (b'US/Central', b'US/Central'),
                    (b'US/Eastern', b'US/Eastern'),
                    (b'US/Hawaii', b'US/Hawaii'),
                    (b'US/Mountain', b'US/Mountain'),
                    (b'US/Pacific', b'US/Pacific'),
                    (b'UTC', b'UTC')
                ], max_length=48, verbose_name='Timezone'
            )
        )
    ]



from django.utils.translation import gettext_lazy as _

from mayan.apps.permissions.classes import PermissionNamespace

namespace = PermissionNamespace(
    label=_(message='Announcements'), name='announcements'
)

permission_announcement_create = namespace.add_permission(
    label=_(message='Create announcements'), name='announcement_create'
)
permission_announcement_delete = namespace.add_permission(
    label=_(message='Delete announcements'), name='announcement_delete'
)
permission_announcement_edit = namespace.add_permission(
    label=_(message='Edit announcements'), name='announcement_edit'
)
permission_announcement_view = namespace.add_permission(
    label=_(message='View announcements'), name='announcement_view'
)



import datetime

from django import forms
from django.utils.timezone import now
from django.utils.translation import gettext_lazy as _

from mayan.apps.common.literals import TIME_DELTA_UNIT_CHOICES
from mayan.apps.views.widgets import NamedMultiWidget


class SplitTimeDeltaWidget(NamedMultiWidget):
    """
    A Widget that splits a timedelta input into two field: one for unit of
    time and another for the amount of units.
    """
    subwidgets = {
        'amount': forms.widgets.NumberInput(
            attrs={
                'maxlength': 4, 'style': 'width: 8em;',
                'placeholder': _(message='Amount')
            }
        ),
        'unit': forms.widgets.Select(
            attrs={'style': 'width: 8em;'}, choices=TIME_DELTA_UNIT_CHOICES
        )
    }

    def decompress(self, value):
        return {
            'amount': None, 'unit': None
        }

    def value_from_datadict(self, querydict, files, name):
        amount = querydict.get(
            '{}_amount'.format(name)
        )
        unit = querydict.get(
            '{}_unit'.format(name)
        )

        if not unit or not amount:
            return now()

        amount = int(amount)

        timedelta = datetime.timedelta(
            **{unit: amount}
        )
        return now() + timedelta



from mayan.apps.testing.tests.base import BaseTestCase

from .mixins import ThemeTestMixin


class ThemeTestCase(ThemeTestMixin, BaseTestCase):
    def test_method_get_absolute_url(self):
        self._create_test_theme()

        self.assertTrue(
            self._test_theme.get_absolute_url()
        )



from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.urls import reverse
from django.utils.translation import gettext_lazy as _

from ..managers import DocumentFilePageManager, ValidDocumentFilePageManager

from .document_file_models import DocumentFile
from .document_file_page_model_mixins import DocumentFilePageBusinessLogicMixin
from .document_version_page_models import DocumentVersionPage
from .model_mixins import PagedModelMixin

__all__ = ('DocumentFilePage', 'DocumentFilePageSearchResult')


class DocumentFilePage(
    DocumentFilePageBusinessLogicMixin, PagedModelMixin, models.Model
):
    """
    Model that describes a document file page
    """
    _paged_model_parent_field = 'document_file'

    document_file = models.ForeignKey(
        on_delete=models.CASCADE, related_name='file_pages', to=DocumentFile,
        verbose_name=_(message='Document file')
    )
    page_number = models.PositiveIntegerField(
        db_index=True, default=1, editable=False,
        verbose_name=_(message='Page number')
    )

    objects = DocumentFilePageManager()
    valid = ValidDocumentFilePageManager()

    class Meta:
        ordering = ('page_number',)
        verbose_name = _(message='Document file page')
        verbose_name_plural = _(message='Document file pages')

    def __str__(self):
        return self.get_label()

    def delete(self, *args, **kwargs):
        """
        When a document file page is deleted also delete any document version
        page referencing it.
        """
        content_type = ContentType.objects.get_for_model(
            model=self
        )
        queryset_document_version_page = DocumentVersionPage.objects.filter(
            content_type=content_type, object_id=self.pk
        )

        self.cache_partition.delete()

        for document_version_page in queryset_document_version_page:
            document_version_page._event_actor = getattr(
                self, '_event_actor', None
            )
            document_version_page.delete()

        super().delete(*args, **kwargs)

    def get_absolute_url(self):
        return reverse(
            viewname='documents:document_file_page_view', kwargs={
                'document_file_page_id': self.pk
            }
        )

    def natural_key(self):
        return (
            self.page_number, self.document_file.natural_key()
        )
    natural_key.dependencies = ['documents.DocumentFile']


class DocumentFilePageSearchResult(DocumentFilePage):
    class Meta:
        ordering = ('document_file__document', 'page_number')
        proxy = True
        verbose_name = _(message='Document file page')
        verbose_name_plural = _(message='Document file pages')



from rest_framework import status

from mayan.apps.rest_api.tests.base import BaseAPITestCase

from ..document_file_actions import (
    DocumentFileActionAppendNewPages, DocumentFileActionNothing
)
from ..events import (
    event_document_version_created, event_document_version_deleted,
    event_document_version_edited, event_document_version_page_created,
    event_document_version_page_deleted
)
from ..permissions import (
    permission_document_version_create, permission_document_version_delete,
    permission_document_version_edit, permission_document_version_view
)

from .mixins.document_file_mixins import DocumentFileTestMixin
from .mixins.document_version_mixins import (
    DocumentVersionAPIViewTestMixin,
    DocumentVersionModificationAPIViewTestMixin, DocumentVersionTestMixin
)


class DocumentVersionModificationAPIViewTestCase(
    DocumentFileTestMixin, DocumentVersionModificationAPIViewTestMixin,
    DocumentVersionTestMixin, BaseAPITestCase
):
    def test_document_version_action_page_append_api_view_no_permission(self):
        self._upload_test_document_file(
            action_name=DocumentFileActionNothing.backend_id
        )

        self._clear_events()

        response = self._request_test_document_version_action_page_append_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self._test_document_version.refresh_from_db()

        self.assertEqual(
            self._test_document_version.pages.count(),
            self._test_document_file_list[0].pages.count()
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_version_action_page_append_api_view_with_access(self):
        self._upload_test_document_file(
            action_name=DocumentFileActionNothing.backend_id
        )

        self.grant_access(
            obj=self._test_document_version,
            permission=permission_document_version_edit
        )

        self._clear_events()

        response = self._request_test_document_version_action_page_append_api_view()
        self.assertEqual(response.status_code, status.HTTP_202_ACCEPTED)

        self._test_document_version.refresh_from_db()

        self.assertEqual(
            self._test_document_version.pages.count(),
            self._test_document_file_list[0].pages.count() + self._test_document_file_list[1].pages.count()
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 4)

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_document_version)
        self.assertEqual(
            events[0].verb, event_document_version_page_deleted.id
        )

        self.assertEqual(
            events[1].action_object, self._test_document_version
        )
        self.assertEqual(events[1].actor, self._test_case_user)
        self.assertEqual(
            events[1].target, self._test_document_version.pages[0]
        )
        self.assertEqual(
            events[1].verb, event_document_version_page_created.id
        )

        self.assertEqual(
            events[2].action_object, self._test_document_version
        )
        self.assertEqual(events[2].actor, self._test_case_user)
        self.assertEqual(
            events[2].target, self._test_document_version.pages[1]
        )
        self.assertEqual(
            events[2].verb, event_document_version_page_created.id
        )

        self.assertEqual(events[3].action_object, self._test_document)
        self.assertEqual(events[3].actor, self._test_case_user)
        self.assertEqual(events[3].target, self._test_document_version)
        self.assertEqual(events[3].verb, event_document_version_edited.id)

    def test_trashed_document_version_action_page_append_api_view_with_access(self):
        self._upload_test_document_file(
            action_name=DocumentFileActionNothing.backend_id
        )

        self.grant_access(
            obj=self._test_document_version,
            permission=permission_document_version_edit
        )

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_document_version_action_page_append_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self._test_document_version.refresh_from_db()

        self.assertEqual(
            self._test_document_version.pages.count(),
            self._test_document_file_list[0].pages.count()
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_version_action_page_reset_api_view_no_permission(self):
        self._upload_test_document_file(
            action_name=DocumentFileActionAppendNewPages.backend_id
        )

        self._clear_events()

        response = self._request_test_document_version_action_page_reset_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self._test_document_version.refresh_from_db()

        self.assertEqual(
            self._test_document_version.pages.count(),
            self._test_document_file_list[0].pages.count() + self._test_document_file_list[1].pages.count()
        )

        self.assertEqual(
            self._test_document_version.pages.all()[0].content_object,
            self._test_document_file_page_list[0]
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_version_action_page_reset_api_view_with_access(self):
        self._upload_test_document_file(
            action_name=DocumentFileActionAppendNewPages.backend_id
        )

        self.grant_access(
            obj=self._test_document_version,
            permission=permission_document_version_edit
        )

        self._clear_events()

        response = self._request_test_document_version_action_page_reset_api_view()
        self.assertEqual(response.status_code, status.HTTP_202_ACCEPTED)

        self._test_document_version.refresh_from_db()

        self.assertEqual(
            self._test_document_version.pages.count(),
            self._test_document_file_list[0].pages.count()
        )

        self.assertEqual(
            self._test_document_version.pages.all()[0].content_object,
            self._test_document_file_page_list[1]
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 4)

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_document_version)
        self.assertEqual(
            events[0].verb, event_document_version_page_deleted.id
        )

        self.assertEqual(events[1].action_object, None)
        self.assertEqual(events[1].actor, self._test_case_user)
        self.assertEqual(events[1].target, self._test_document_version)
        self.assertEqual(
            events[1].verb, event_document_version_page_deleted.id
        )

        self.assertEqual(
            events[2].action_object, self._test_document_version
        )
        self.assertEqual(events[2].actor, self._test_case_user)
        self.assertEqual(
            events[2].target, self._test_document_version.pages[0]
        )
        self.assertEqual(
            events[2].verb, event_document_version_page_created.id
        )

        self.assertEqual(events[3].action_object, self._test_document)
        self.assertEqual(events[3].actor, self._test_case_user)
        self.assertEqual(events[3].target, self._test_document_version)
        self.assertEqual(events[3].verb, event_document_version_edited.id)

    def test_trashed_document_version_action_page_reset_api_view_with_access(self):
        self._upload_test_document_file(
            action_name=DocumentFileActionAppendNewPages.backend_id
        )

        self.grant_access(
            obj=self._test_document_version,
            permission=permission_document_version_edit
        )

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_document_version_action_page_reset_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self._test_document_version.refresh_from_db()

        self.assertEqual(
            self._test_document_version.pages.count(),
            self._test_document_file_list[0].pages.count() + self._test_document_file_list[1].pages.count()
        )

        self.assertEqual(
            self._test_document_version.pages.all()[0].content_object,
            self._test_document_file_page_list[0]
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)


class DocumentVersionAPIViewTestCase(
    DocumentVersionAPIViewTestMixin, DocumentVersionTestMixin,
    BaseAPITestCase
):
    def test_document_version_create_api_view_no_permission(self):
        document_version_count = self._test_document.versions.count()

        self._clear_events()

        response = self._request_test_document_version_create_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.assertEqual(
            self._test_document.versions.count(), document_version_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_version_create_api_view_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_version_create
        )

        document_version_count = self._test_document.versions.count()

        self._clear_events()

        response = self._request_test_document_version_create_api_view()
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

        self.assertEqual(
            self._test_document.versions.count(), document_version_count + 1
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, self._test_document)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_document_version)
        self.assertEqual(events[0].verb, event_document_version_created.id)

    def test_trashed_document_version_create_api_view_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_version_create
        )

        document_version_count = self._test_document.versions.count()

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_document_version_create_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.assertEqual(
            self._test_document.versions.count(), document_version_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_version_delete_api_view_no_permission(self):
        document_version_count = self._test_document.versions.count()

        self._clear_events()

        response = self._request_test_document_version_delete_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.assertEqual(
            self._test_document.versions.count(), document_version_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_version_delete_api_view_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_version_delete
        )

        document_version_count = self._test_document.versions.count()

        self._clear_events()

        response = self._request_test_document_version_delete_api_view()
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

        self.assertEqual(
            self._test_document.versions.count(), document_version_count - 1
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_document)
        self.assertEqual(events[0].verb, event_document_version_deleted.id)

    def test_trashed_document_version_delete_api_view_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_version_delete
        )

        document_version_count = self._test_document.versions.count()

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_document_version_delete_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.assertEqual(
            self._test_document.versions.count(), document_version_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_version_detail_api_view_no_permission(self):
        self._clear_events()

        response = self._request_test_document_version_detail_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_version_detail_api_view_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_version_view
        )

        self._clear_events()

        response = self._request_test_document_version_detail_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertEqual(
            response.data['id'], self._test_document.version_active.id
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_document_version_detail_api_view_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_version_view
        )

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_document_version_detail_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_version_edit_via_patch_api_view_no_permission(self):
        document_version_comment = self._test_document.version_active.comment

        self._clear_events()

        response = self._request_test_document_version_edit_via_patch_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self._test_document.version_active.refresh_from_db()
        self.assertEqual(
            self._test_document.version_active.comment,
            document_version_comment
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_version_edit_via_patch_api_view_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_version_edit
        )

        document_version_comment = self._test_document.version_active.comment

        self._clear_events()

        response = self._request_test_document_version_edit_via_patch_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self._test_document.version_active.refresh_from_db()
        self.assertNotEqual(
            self._test_document.version_active.comment,
            document_version_comment
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, self._test_document)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_document_version)
        self.assertEqual(events[0].verb, event_document_version_edited.id)

    def test_trashed_document_version_edit_via_patch_api_view_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_version_edit
        )

        document_version_comment = self._test_document.version_active.comment

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_document_version_edit_via_patch_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self._test_document.version_active.refresh_from_db()
        self.assertEqual(
            self._test_document.version_active.comment,
            document_version_comment
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_version_edit_via_put_api_view_no_permission(self):
        document_version_comment = self._test_document.version_active.comment

        self._clear_events()

        response = self._request_test_document_version_edit_via_put_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self._test_document.version_active.refresh_from_db()
        self.assertEqual(
            self._test_document.version_active.comment,
            document_version_comment
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_version_edit_via_put_api_view_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_version_edit
        )

        document_version_comment = self._test_document.version_active.comment

        self._clear_events()

        response = self._request_test_document_version_edit_via_put_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self._test_document.version_active.refresh_from_db()
        self.assertNotEqual(
            self._test_document.version_active.comment,
            document_version_comment
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, self._test_document)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_document_version)
        self.assertEqual(events[0].verb, event_document_version_edited.id)

    def test_trashed_document_version_edit_via_put_api_view_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_version_edit
        )

        document_version_comment = self._test_document.version_active.comment

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_document_version_edit_via_put_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self._test_document.version_active.refresh_from_db()
        self.assertEqual(
            self._test_document.version_active.comment,
            document_version_comment
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_version_list_api_view_no_permission(self):
        self._clear_events()

        response = self._request_test_document_version_list_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_version_list_api_view_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_version_view
        )

        self._clear_events()

        response = self._request_test_document_version_list_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertEqual(
            response.data['results'][0]['id'],
            self._test_document.version_active.id
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_document_version_list_api_view_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_version_view
        )

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_document_version_list_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)


class DocumentVersionBusinessLogicAPIViewTestCase(
    DocumentVersionAPIViewTestMixin, DocumentVersionTestMixin,
    BaseAPITestCase
):
    def test_document_version_multiple_active_via_put_api_view_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_version_edit
        )

        self._create_test_document_version()

        self._clear_events()

        response = self._request_test_document_version_edit_via_put_api_view(
            extra_view_kwargs={
                'document_version_id': self._test_document_version_list[1].pk
            }
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self._test_document_version_list[0].refresh_from_db()
        self._test_document_version_list[1].refresh_from_db()

        self.assertEqual(self._test_document_version_list[0].active, False)
        self.assertEqual(self._test_document_version_list[1].active, True)

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, self._test_document)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_document_version)
        self.assertEqual(events[0].verb, event_document_version_edited.id)



from django.utils.translation import gettext_lazy as _

from mayan.apps.navigation.classes import Link
from mayan.apps.navigation.utils import factory_condition_queryset_access

from .icons import (
    icon_document_file_content_detail,
    icon_document_file_content_single_delete,
    icon_document_file_content_multiple_delete,
    icon_document_file_content_download,
    icon_document_file_page_content_detail,
    icon_document_file_parsing_multiple_submit,
    icon_document_type_parsing_settings,
    icon_document_file_parsing_single_submit,
    icon_document_type_parsing_submit
)
from .permissions import (
    permission_document_file_content_view,
    permission_document_type_parsing_setup, permission_document_file_parse
)

# Document file

link_document_file_content_detail = Link(
    args='resolved_object.id', icon=icon_document_file_content_detail,
    permission=permission_document_file_content_view, text=_(message='Content'),
    view='document_parsing:document_file_content_view'
)
link_document_file_content_single_delete = Link(
    args='resolved_object.id', icon=icon_document_file_content_single_delete,
    permission=permission_document_file_parse,
    text=_(message='Delete parsed content'),
    view='document_parsing:document_file_content_single_delete',
)
link_document_file_content_multiple_delete = Link(
    icon=icon_document_file_content_multiple_delete,
    text=_(message='Delete parsed content'),
    view='document_parsing:document_file_content_multiple_delete',
)
link_document_file_content_download = Link(
    args='resolved_object.id', icon=icon_document_file_content_download,
    permission=permission_document_file_content_view,
    text=_(message='Download content'),
    view='document_parsing:document_file_content_download'
)
link_document_file_page_content_detail = Link(
    args='resolved_object.id',
    icon=icon_document_file_page_content_detail,
    permission=permission_document_file_content_view, text=_(message='Content'),
    view='document_parsing:document_file_page_content_view'
)
link_document_file_parsing_multiple_submit = Link(
    icon=icon_document_file_parsing_multiple_submit,
    text=_(message='Submit for parsing'),
    view='document_parsing:document_file_parsing_multiple_submit'
)
link_document_file_parsing_single_submit = Link(
    args='resolved_object.id',
    icon=icon_document_file_parsing_single_submit,
    permission=permission_document_file_parse,
    text=_(message='Submit for parsing'),
    view='document_parsing:document_file_parsing_single_submit'
)

# Document type

link_document_type_parsing_settings = Link(
    args='resolved_object.id',
    icon=icon_document_type_parsing_settings,
    permission=permission_document_type_parsing_setup,
    text=_(message='Setup parsing'),
    view='document_parsing:document_type_parsing_settings'
)
link_document_type_parsing_submit = Link(
    condition=factory_condition_queryset_access(
        app_label='documents', model_name='DocumentType',
        object_permission=permission_document_type_parsing_setup
    ),
    icon=icon_document_type_parsing_submit,
    text=_(message='Parse documents per type'),
    view='document_parsing:document_type_submit'
)



import base64
import io

from django import forms
from django.utils.encoding import force_str


class Base64ImageWidget(forms.widgets.Widget):
    template_name = 'converter/forms/widgets/base64_image.html'

    def format_value(self, value):
        if value == '' or value is None:
            return None
        else:
            with io.BytesIO() as output:
                value.save(format='PNG', stream=output)
                image = output.getvalue()
                url = 'data:image/png;charset=utf-8;base64,{}'.format(
                    force_str(
                        s=base64.b64encode(s=image)
                    )
                )

                return url


class ImageWidget(forms.widgets.Widget):
    template_name = 'converter/forms/widgets/widget_image.html'

    def format_value(self, value):
        if value == '' or value is None:
            return None
        return value



from mayan.apps.dynamic_search.tests.mixins.base import SearchTestMixin

from ..permissions import permission_document_file_view
from ..search import (
    search_model_document_file, search_model_document_file_page
)

from .base import GenericDocumentViewTestCase
from .literals import TEST_DOCUMENT_FILE_COMMENT


class DocumentFileSearchTestCase(
    SearchTestMixin, GenericDocumentViewTestCase
):
    _test_search_model = search_model_document_file
    auto_upload_test_document = False

    def setUp(self):
        super().setUp()
        self._upload_test_document(
            document_file_attributes={'comment': TEST_DOCUMENT_FILE_COMMENT}
        )

    def test_search_model_document_file_by_checksum_no_permission(self):
        self._clear_events()

        queryset = self._do_test_search(
            query={
                'checksum': self._test_document_file.checksum
            }
        )
        self.assertTrue(self._test_document_file not in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_search_model_document_file_by_checksum_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_file_view
        )

        self._clear_events()

        queryset = self._do_test_search(
            query={
                'checksum': self._test_document_file.checksum
            }
        )
        self.assertTrue(self._test_document_file in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_search_model_document_file_by_checksum_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_file_view
        )

        self._test_document.delete()

        self._clear_events()

        queryset = self._do_test_search(
            query={
                'checksum': self._test_document_file.checksum
            }
        )
        self.assertTrue(self._test_document_file not in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_search_model_document_file_by_comment_no_permission(self):
        self._clear_events()

        queryset = self._do_test_search(
            query={
                'comment': self._test_document_file.comment
            }
        )
        self.assertTrue(self._test_document_file not in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_search_model_document_file_by_comment_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_file_view
        )

        self._clear_events()

        queryset = self._do_test_search(
            query={
                'comment': self._test_document_file.comment
            }
        )
        self.assertTrue(self._test_document_file in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_search_model_document_file_by_comment_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_file_view
        )

        self._test_document.delete()

        self._clear_events()

        queryset = self._do_test_search(
            query={
                'comment': self._test_document_file.comment
            }
        )
        self.assertTrue(self._test_document_file not in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_search_model_document_file_by_filename_no_permission(self):
        self._clear_events()

        queryset = self._do_test_search(
            query={
                'filename': self._test_document_file.filename
            }
        )
        self.assertTrue(self._test_document_file not in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_search_model_document_file_by_filename_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_file_view
        )

        self._clear_events()

        queryset = self._do_test_search(
            query={
                'filename': self._test_document_file.filename
            }
        )
        self.assertTrue(self._test_document_file in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_search_model_document_file_by_filename_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_file_view
        )

        self._test_document.delete()

        self._clear_events()

        queryset = self._do_test_search(
            query={
                'filename': self._test_document_file.filename
            }
        )
        self.assertTrue(self._test_document_file not in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_search_model_document_file_by_mime_type_no_permission(self):
        self._clear_events()

        queryset = self._do_test_search(
            query={
                'mimetype': self._test_document_file.mimetype
            }
        )
        self.assertTrue(self._test_document_file not in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_search_model_document_file_by_mime_type_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_file_view
        )

        self._clear_events()

        queryset = self._do_test_search(
            query={
                'mimetype': self._test_document_file.mimetype
            }
        )
        self.assertTrue(self._test_document_file in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_search_model_document_file_by_mime_type_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_file_view
        )

        self._test_document.delete()

        self._clear_events()

        queryset = self._do_test_search(
            query={
                'mimetype': self._test_document_file.mimetype
            }
        )
        self.assertTrue(self._test_document_file not in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_search_model_document_file_by_document_description_no_permission(self):
        self._clear_events()

        queryset = self._do_test_search(
            query={
                'document__description': self._test_document.description
            }
        )
        self.assertTrue(self._test_document_file not in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_search_model_document_file_by_document_description_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_file_view
        )

        self._clear_events()

        queryset = self._do_test_search(
            query={
                'document__description': self._test_document.description
            }
        )
        self.assertTrue(self._test_document_file in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_search_model_document_file_by_document_description_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_file_view
        )

        self._test_document.delete()

        self._clear_events()

        queryset = self._do_test_search(
            query={
                'document__description': self._test_document.description
            }
        )
        self.assertTrue(self._test_document_file not in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_search_model_document_file_by_document_label_no_permission(self):
        self._clear_events()

        queryset = self._do_test_search(
            query={
                'document__label': self._test_document.label
            }
        )
        self.assertTrue(self._test_document_file not in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_search_model_document_file_by_document_label_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_file_view
        )

        self._clear_events()

        queryset = self._do_test_search(
            query={
                'document__label': self._test_document.label
            }
        )
        self.assertTrue(self._test_document_file in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_search_model_document_file_by_document_label_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_file_view
        )

        self._test_document.delete()

        self._clear_events()

        queryset = self._do_test_search(
            query={
                'document__label': self._test_document.label
            }
        )
        self.assertTrue(self._test_document_file not in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_search_model_document_file_by_document_uuid_no_permission(self):
        self._clear_events()

        queryset = self._do_test_search(
            query={
                'document__uuid': str(self._test_document.uuid)
            }
        )
        self.assertTrue(self._test_document_file not in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_search_model_document_file_by_document_uuid_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_file_view
        )

        self._clear_events()

        queryset = self._do_test_search(
            query={
                'document__uuid': str(self._test_document.uuid)
            }
        )
        self.assertTrue(self._test_document_file in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_search_model_document_file_by_document_uuid_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_file_view
        )

        self._test_document.delete()

        self._clear_events()

        queryset = self._do_test_search(
            query={
                'document__uuid': str(self._test_document.uuid)
            }
        )
        self.assertTrue(self._test_document_file not in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_search_model_document_file_by_document_type_label_no_permission(self):
        self._clear_events()

        queryset = self._do_test_search(
            query={
                'document__document_type__label': self._test_document_type.label
            }
        )
        self.assertTrue(self._test_document_file not in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_search_model_document_file_by_document_type_label_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_file_view
        )

        self._clear_events()

        queryset = self._do_test_search(
            query={
                'document__document_type__label': self._test_document_type.label
            }
        )
        self.assertTrue(self._test_document_file in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_search_model_document_file_by_document_type_label_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_file_view
        )

        self._test_document.delete()

        self._clear_events()

        queryset = self._do_test_search(
            query={
                'document__document_type__label': self._test_document_type.label
            }
        )
        self.assertTrue(self._test_document_file not in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)


class DocumentFilePageSearchTestCase(
    SearchTestMixin, GenericDocumentViewTestCase
):
    _test_search_model = search_model_document_file_page
    auto_upload_test_document = False

    def setUp(self):
        super().setUp()
        self._upload_test_document()

    def test_search_model_document_file_page_by_document_label_no_permission(self):
        self._clear_events()

        queryset = self._do_test_search(
            query={
                'document_file__document__label': self._test_document.label
            }
        )
        self.assertTrue(self._test_document_file_page not in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_search_model_document_file_page_by_document_label_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_file_view
        )

        self._clear_events()

        queryset = self._do_test_search(
            query={
                'document_file__document__label': self._test_document.label
            }
        )
        self.assertTrue(self._test_document_file_page in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_search_model_document_file_page_by_document_label_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_file_view
        )

        self._test_document.delete()

        self._clear_events()

        queryset = self._do_test_search(
            query={
                'document_file__document__label': self._test_document.label
            }
        )
        self.assertTrue(self._test_document_file_page not in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_search_model_document_file_page_by_document_file_checksum_no_permission(self):
        self._clear_events()

        queryset = self._do_test_search(
            query={
                'document_file__checksum': self._test_document_file.checksum
            }
        )
        self.assertTrue(self._test_document_file_page not in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_search_model_document_file_page_by_document_file_checksum_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_file_view
        )

        self._clear_events()

        queryset = self._do_test_search(
            query={
                'document_file__checksum': self._test_document_file.checksum
            }
        )
        self.assertTrue(self._test_document_file_page in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_search_model_document_file_page_by_document_file_checksum_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_file_view
        )

        self._test_document.delete()

        self._clear_events()

        queryset = self._do_test_search(
            query={
                'document_file__checksum': self._test_document_file.checksum
            }
        )
        self.assertTrue(self._test_document_file_page not in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_search_model_document_file_page_by_document_uuid_no_permission(self):
        self._clear_events()

        queryset = self._do_test_search(
            query={
                'document_file__document__uuid': str(
                    self._test_document.uuid
                )
            }
        )
        self.assertTrue(self._test_document_file_page not in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_search_model_document_file_page_by_document_uuid_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_file_view
        )

        self._clear_events()

        queryset = self._do_test_search(
            query={
                'document_file__document__uuid': str(
                    self._test_document.uuid
                )
            }
        )
        self.assertTrue(self._test_document_file_page in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_search_model_document_file_page_by_document_uuid_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_file_view
        )

        self._test_document.delete()

        self._clear_events()

        queryset = self._do_test_search(
            query={
                'document_file__document__uuid': str(
                    self._test_document.uuid
                )
            }
        )
        self.assertTrue(self._test_document_file_page not in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_search_model_document_file_page_by_document_type_label_no_permission(self):
        self._clear_events()

        queryset = self._do_test_search(
            query={
                'document_file__document__document_type__label': self._test_document_type.label
            }
        )
        self.assertTrue(self._test_document_file_page not in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_search_model_document_file_page_by_document_type_label_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_file_view
        )

        self._clear_events()

        queryset = self._do_test_search(
            query={
                'document_file__document__document_type__label': self._test_document_type.label
            }
        )
        self.assertTrue(self._test_document_file_page in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_search_model_document_file_page_by_document_type_label_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_file_view
        )

        self._test_document.delete()

        self._clear_events()

        queryset = self._do_test_search(
            query={
                'document_file__document__document_type__label': self._test_document_type.label
            }
        )
        self.assertTrue(self._test_document_file_page not in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)



from django.apps import apps

from .classes import Permission


def handler_permission_initialize(**kwargs):
    Permission.load_modules()


def handler_purge_permissions(**kwargs):
    StoredPermission = apps.get_model(
        app_label='permissions', model_name='StoredPermission'
    )

    StoredPermission.objects.purge_obsolete()



from django.contrib.auth.models import AnonymousUser
from django.core.exceptions import PermissionDenied

from mayan.apps.testing.tests.base import BaseTestCase
from mayan.apps.user_management.tests.mixins import GroupTestMixin

from ..classes import Permission, PermissionNamespace
from ..models import StoredPermission

from .literals import (
    TEST_INVALID_PERMISSION_NAME, TEST_INVALID_PERMISSION_NAMESPACE_NAME,
    TEST_PERMISSION_LABEL, TEST_PERMISSION_NAME,
    TEST_PERMISSION_NAMESPACE_LABEL, TEST_PERMISSION_NAMESPACE_NAME
)
from .mixins import RoleTestMixin


class PermissionTestCase(GroupTestMixin, RoleTestMixin, BaseTestCase):
    def setUp(self):
        super().setUp()
        self._create_test_user()
        self._create_test_group()
        self._create_test_role()
        self._create_test_permission()

    def test_no_permission(self):
        with self.assertRaises(expected_exception=PermissionDenied):
            Permission.check_user_permission(
                permission=self._test_permission, user=self._test_user
            )

    def test_with_permissions(self):
        self._test_group.user_set.add(self._test_user)
        self._test_role.grant(permission=self._test_permission)
        self._test_role.groups.add(self._test_group)

        try:
            Permission.check_user_permission(
                permission=self._test_permission, user=self._test_user
            )
        except PermissionDenied:
            self.fail('PermissionDenied exception was not expected.')

    def test_anonymous_user_permissions(self):
        self.auto_login_user = False
        test_anonymous_user = AnonymousUser()

        with self.assertRaises(expected_exception=PermissionDenied):
            Permission.check_user_permission(
                permission=self._test_permission, user=test_anonymous_user
            )


class RoleModelTestCase(RoleTestMixin, BaseTestCase):
    def test_method_get_absolute_url(self):
        self._create_test_role()

        self.assertTrue(
            self._test_role.get_absolute_url()
        )


class StoredPermissionManagerTestCase(BaseTestCase):
    create_test_case_super_user = False
    create_test_case_user = False

    def test_purge_obsolete_with_invalid(self):
        StoredPermission.objects.create(
            namespace=TEST_INVALID_PERMISSION_NAMESPACE_NAME,
            name=TEST_INVALID_PERMISSION_NAME
        )

        permission_count = StoredPermission.objects.count()

        StoredPermission.objects.purge_obsolete()

        self.assertEqual(
            StoredPermission.objects.count(), permission_count - 1
        )

    def test_purge_obsolete_with_valid(self):
        test_permission_namespace = PermissionNamespace(
            label=TEST_PERMISSION_NAMESPACE_LABEL,
            name=TEST_PERMISSION_NAMESPACE_NAME
        )
        test_permission = test_permission_namespace.add_permission(
            label=TEST_PERMISSION_LABEL, name=TEST_PERMISSION_NAME
        )
        test_permission.stored_permission

        permission_count = StoredPermission.objects.count()

        StoredPermission.objects.purge_obsolete()

        self.assertEqual(
            StoredPermission.objects.count(), permission_count
        )



from django.utils.translation import gettext_lazy as _

DEFAULT_DOCUMENT_FILE_DOWNLOAD_MESSAGE_BODY = _(
    'The document files have been compressed '
    'and are available for download using the '
    'link: {{ download_url }} or from '
    'the downloads area ({{ download_list_url }}).'
)
DEFAULT_DOCUMENT_FILE_DOWNLOAD_MESSAGE_SUBJECT = _(
    'Document files ready for download.'
)



from pathlib import Path
from unittest import skip

from django.core.files.base import ContentFile

from mayan.apps.common.tests.literals import (
    TEST_BINARY_CONTENT, TEST_FILE_NAME
)
from mayan.apps.mime_types.tests.mixins import MIMETypeBackendMixin
from mayan.apps.storage.utils import fs_cleanup, mkdtemp
from mayan.apps.testing.tests.base import BaseTestCase

from ..backends.compressedstorage import ZipCompressedPassthroughStorage
from ..backends.encryptedstorage import EncryptedPassthroughStorage


class EncryptedPassthroughStorageTestCase(
    MIMETypeBackendMixin, BaseTestCase
):
    def setUp(self):
        super().setUp()
        self.temporary_directory = mkdtemp()

    def tearDown(self):
        fs_cleanup(filename=self.temporary_directory)
        super().tearDown()

    def test_file_save_and_load(self):
        storage = EncryptedPassthroughStorage(
            password='testpassword',
            next_storage_backend_arguments={
                'location': self.temporary_directory
            }
        )

        test_file_name = storage.save(
            name=TEST_FILE_NAME, content=ContentFile(
                content=TEST_BINARY_CONTENT
            )
        )

        path_file = Path(self.temporary_directory) / test_file_name

        with path_file.open(mode='rb') as file_object:
            self.assertEqual(
                self.mime_type_backend.get_mime_type(
                    file_object=file_object
                ), ('application/octet-stream', 'binary')
            )

        with storage.open(name=TEST_FILE_NAME, mode='rb') as file_object:
            self.assertEqual(
                file_object.read(), TEST_BINARY_CONTENT
            )

        with storage.open(name=TEST_FILE_NAME, mode='rb') as file_object:
            self.assertEqual(
                file_object.read(1), TEST_BINARY_CONTENT[0:1]
            )

        with storage.open(name=TEST_FILE_NAME, mode='rb') as file_object:
            self.assertEqual(
                file_object.read(999), TEST_BINARY_CONTENT
            )


class ZipCompressedPassthroughStorageTestCase(
    MIMETypeBackendMixin, BaseTestCase
):
    def setUp(self):
        super().setUp()
        self.temporary_directory = mkdtemp()

    def tearDown(self):
        fs_cleanup(filename=self.temporary_directory)
        super().tearDown()

    @skip(
        reason='get_mime_type() is not recognizing deflated Zips some times.'
    )
    def test_file_save_and_load(self):
        storage = ZipCompressedPassthroughStorage(
            next_storage_backend_arguments={
                'location': self.temporary_directory
            }
        )

        test_file_name = storage.save(
            name=TEST_FILE_NAME, content=ContentFile(
                content=TEST_BINARY_CONTENT
            )
        )

        path_file = Path(self.temporary_directory) / test_file_name

        with path_file.open(mode='rb') as file_object:
            self.assertTrue(
                self.mime_type_backend.get_mime_type(
                    file_object=file_object, mime=False, mime_type_only=True
                )[0].startswith('Zip archive data, made by v2.0')
            )

        with path_file.open(mode='rb') as file_object:
            self.assertNotEqual(
                file_object.read(), TEST_BINARY_CONTENT
            )

        with storage.open(name=TEST_FILE_NAME, mode='rb') as file_object:
            self.assertEqual(file_object.read(), TEST_BINARY_CONTENT)


class CombinationPassthroughStorageTestCase(
    MIMETypeBackendMixin, BaseTestCase
):
    def setUp(self):
        super().setUp()
        self.temporary_directory = mkdtemp()

    def tearDown(self):
        fs_cleanup(filename=self.temporary_directory)
        super().tearDown()

    def test_file_save_and_load(self):
        storage = EncryptedPassthroughStorage(
            password='testpassword',
            next_storage_backend='mayan.apps.storage.backends.compressedstorage.ZipCompressedPassthroughStorage',
            next_storage_backend_arguments={
                'next_storage_backend_arguments': {
                    'location': self.temporary_directory
                }
            }
        )

        test_file_name = storage.save(
            name=TEST_FILE_NAME, content=ContentFile(
                content=TEST_BINARY_CONTENT
            )
        )

        path_file = Path(self.temporary_directory) / test_file_name

        with path_file.open(mode='rb') as file_object:
            self.assertEqual(
                self.mime_type_backend.get_mime_type(
                    file_object=file_object
                ), ('application/zip', 'binary')
            )

        with path_file.open(mode='rb') as file_object:
            self.assertNotEqual(
                file_object.read(), TEST_BINARY_CONTENT
            )

        with storage.open(name=TEST_FILE_NAME, mode='rb') as file_object:
            self.assertEqual(
                file_object.read(), TEST_BINARY_CONTENT
            )



from rest_framework import status

from mayan.apps.documents.tests.mixins.document_mixins import DocumentTestMixin
from mayan.apps.rest_api.tests.base import BaseAPITestCase

from ..permissions import permission_document_type_ocr_setup

from .mixins import DocumentTypeOCRSettingsAPIViewTestMixin


class DocumentTypeOCRSettingsAPIViewTestCase(
    DocumentTestMixin, DocumentTypeOCRSettingsAPIViewTestMixin,
    BaseAPITestCase
):
    auto_upload_test_document = False

    def test_document_type_ocr_settings_details_api_view_no_permission(self):
        self._clear_events()

        response = self._request_test_document_type_ocr_settings_details_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_type_ocr_settings_details_api_view_with_access(self):
        self.grant_access(
            obj=self._test_document_type,
            permission=permission_document_type_ocr_setup
        )

        self._clear_events()

        response = self._request_test_document_type_ocr_settings_details_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(
            response.data, {'auto_ocr': False}
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_type_ocr_settings_patch_api_view_no_permission(self):
        self._clear_events()

        response = self._request_test_document_type_ocr_settings_patch_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_type_ocr_settings_patch_api_view_with_access(self):
        self.grant_access(
            obj=self._test_document_type,
            permission=permission_document_type_ocr_setup
        )

        self._clear_events()

        response = self._request_test_document_type_ocr_settings_patch_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(
            response.data, {'auto_ocr': True}
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_type_ocr_settings_put_api_view_no_permission(self):
        self._clear_events()

        response = self._request_test_document_type_ocr_settings_put_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_type_ocr_settings_put_api_view_with_access(self):
        self.grant_access(
            obj=self._test_document_type,
            permission=permission_document_type_ocr_setup
        )

        self._clear_events()

        response = self._request_test_document_type_ocr_settings_put_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(
            response.data, {'auto_ocr': True}
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)



from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ImproperlyConfigured

from rest_framework import status
from rest_framework.generics import get_object_or_404
from rest_framework.response import Response
from rest_framework.settings import api_settings

from mayan.apps.databases.utils import check_queryset
from mayan.apps.views.view_mixins import ExternalObjectBaseMixin

from .literals import (
    QUERY_FIELD_EXCLUDE_PARAMETER, QUERY_FIELD_ONLY_PARAMETER
)


class AsymmetricSerializerAPIViewMixin:
    _write_methods = ('PATCH', 'POST', 'PUT')
    read_serializer_class = None
    write_serializer_class = None

    def get_read_serializer_class(self):
        if not self.read_serializer_class:
            raise ImproperlyConfigured(
                'View {} must specify a `read_serializer_class`.'.format(
                    self.__class__.__name__
                )
            )
        else:
            return self.read_serializer_class

    def get_serializer_class(self):
        if getattr(self, 'serializer_class', None):
            raise ImproperlyConfigured(
                'View {} can not use `AsymmetricSerializerAPIViewMixin` '
                'while also specifying a `serializer_class`.'.format(
                    self.__class__.__name__
                )
            )

        if self.request.method in self._write_methods:
            return self.get_write_serializer_class()
        else:
            return self.get_read_serializer_class()

    def get_write_serializer_class(self):
        if not self.write_serializer_class:
            raise ImproperlyConfigured(
                'View {} must specify a `write_serializer_class`.'.format(
                    self.__class__.__name__
                )
            )
        else:
            return self.write_serializer_class


class CheckQuerysetAPIViewMixin:
    def get_source_queryset(self, *args, **kwargs):
        queryset = super().get_source_queryset(*args, **kwargs)
        return check_queryset(view=self, queryset=queryset)


class ContentTypeAPIViewMixin:
    """
    This mixin makes it easier for API views to retrieve a content type from
    the URL pattern.
    """
    content_type_url_kw_args = {
        'app_label': 'app_label',
        'model_name': 'model_name'
    }

    def get_content_type(self):
        return get_object_or_404(
            queryset=ContentType, app_label=self.kwargs.get(
                self.content_type_url_kw_args['app_label']
            ), model=self.kwargs.get(
                self.content_type_url_kw_args['model_name']
            )
        )


class DynamicFieldListAPIViewMixin:
    def get_serializer_extra_context(self):
        context = super().get_serializer_extra_context()

        if self.request.method == 'GET':
            context.update(
                {
                    'fields_exclude': self.request.query_params.get(
                        QUERY_FIELD_EXCLUDE_PARAMETER, ''
                    ),
                    'fields_only': self.request.query_params.get(
                        QUERY_FIELD_ONLY_PARAMETER, ''
                    )
                }
            )

        return context


class ExternalObjectAPIViewMixin(ExternalObjectBaseMixin):
    """
    Override get_external_object to use REST API get_object_or_404.
    """
    def get_external_object(self):
        return get_object_or_404(
            queryset=self.get_external_object_queryset_filtered(),
            **self.get_pk_url_kwargs()
        )

    def get_external_object_permission(self):
        mayan_external_object_permission_map = getattr(
            self, 'mayan_external_object_permission_map', {}
        )

        permission = mayan_external_object_permission_map.get(
            self.request.method, None
        )

        return permission

    def get_serializer_extra_context(self):
        """
        Add the external object to the serializer context. Useful for the
        create action when there is no instance available.
        """
        context = super().get_serializer_extra_context()
        if self.kwargs:
            context['external_object'] = self.get_external_object()

        return context


class ExternalContentTypeObjectAPIViewMixin(
    ContentTypeAPIViewMixin, ExternalObjectAPIViewMixin
):
    """
    Mixin to retrieve an external object by content type from the URL pattern.
    """
    external_object_pk_url_kwarg = 'object_id'

    def get_external_object_queryset(self):
        self.content_type = self.get_content_type()
        self.external_object_class = self.content_type.model_class()
        return super().get_external_object_queryset()


class InstanceExtraDataAPIViewMixin:
    def perform_create(self, serializer):
        if hasattr(self, 'get_instance_extra_data'):
            serializer.validated_data['_instance_extra_data'] = self.get_instance_extra_data()

        return super().perform_create(serializer=serializer)

    def perform_destroy(self, instance):
        if hasattr(self, 'get_instance_extra_data'):
            for key, value in self.get_instance_extra_data().items():
                setattr(instance, key, value)

        return super().perform_destroy(instance=instance)

    def perform_update(self, serializer):
        if hasattr(self, 'get_instance_extra_data'):
            for key, value in self.get_instance_extra_data().items():
                serializer.validated_data[key] = value

        return super().perform_update(serializer=serializer)


class SchemaInspectionAPIViewMixin:
    def get_serializer_context(self, *args, **kwargs):
        if getattr(self, 'swagger_fake_view', False):
            return {'request': self.request}
        else:
            return super().get_serializer_context(*args, **kwargs)


class QuerySetOverrideCheckAPIViewMixin:
    source_queryset = None

    def __init__(self, *args, **kwargs):
        result = super().__init__(*args, **kwargs)

        queryset = getattr(self, 'queryset', None)

        if queryset is not None:
            raise ImproperlyConfigured(
                '%(cls)s is overloading the `queryset` property. '
                'Subclasses should use the `source_queryset` property '
                'instead. ' % {
                    'cls': self.__class__.mro()[0].__name__
                }
            )

        if self.__class__.mro()[0].get_queryset != QuerySetOverrideCheckAPIViewMixin.get_queryset:
            raise ImproperlyConfigured(
                '%(cls)s is overloading the `get_queryset` method. '
                'Subclasses should implement the `get_source_queryset` '
                'method instead. ' % {
                    'cls': self.__class__.mro()[0].__name__
                }
            )

        return result

    def get_source_queryset(self):
        if self.source_queryset is None:
            if self.model:
                return self.model._default_manager.all()
            else:
                raise ImproperlyConfigured(
                    '%(cls)s is missing a QuerySet. Define '
                    '`%(cls)s.model`, `%(cls)s.source_queryset`, or '
                    'override `%(cls)s.get_source_queryset()`.' % {
                        'cls': self.__class__.__name__
                    }
                )

        return self.source_queryset

    def get_queryset(self):
        if getattr(self, 'swagger_fake_view', False):
            return []
        else:
            try:
                return super().get_queryset()
            except AssertionError:
                self.queryset = self.get_source_queryset()
                return super().get_queryset()
            except ImproperlyConfigured:
                self.queryset = self.get_source_queryset()
                return super().get_queryset()


class SerializerActionAPIViewMixin:
    serializer_action_name = None

    def get_success_headers(self, data):
        try:
            return {
                'Location': str(
                    data[api_settings.URL_FIELD_NAME]
                )
            }
        except (TypeError, KeyError):
            return {}

    def perform_serializer_action(self, serializer):
        getattr(serializer, self.serializer_action_name)()

    def post(self, request, *args, **kwargs):
        return self.serializer_action(request=request, *args, **kwargs)

    def serializer_action(self, request, *args, **kwargs):
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        self.perform_serializer_action(serializer=serializer)
        headers = self.get_success_headers(serializer.data)
        return Response(
            serializer.data, headers=headers, status=status.HTTP_200_OK
        )


class SerializerExtraContextAPIViewMixin:
    def get_serializer_context(self):
        context = super().get_serializer_context()

        if not getattr(self, 'swagger_fake_view', False):
            context.update(
                self.get_serializer_extra_context()
            )

        return context

    def get_serializer_extra_context(self):
        return {}


class SuccessHeadersAPIViewMixin:
    def get_success_headers(self, data):
        try:
            return {
                'Location': str(
                    data[api_settings.URL_FIELD_NAME]
                )
            }
        except (TypeError, KeyError):
            return {}



from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('document_states', '0021_auto_20200624_0719')
    ]

    operations = [
        migrations.AddField(
            model_name='workflow',
            name='auto_launch',
            field=models.BooleanField(
                default=True, verbose_name='Launch workflow when document '
                'is created.'
            )
        )
    ]



from django.conf import settings
from django.contrib.auth.views import PasswordResetConfirmView
from django.test import override_settings

from ..classes import AuthenticationBackend
from ..literals import USER_IMPERSONATE_VARIABLE_ID

from .literals import PATH_AUTHENTICATION_BACKEND_USERNAME


class LoginViewTestMixin:
    def _request_authenticated_view(self):
        return self.get(path=self.authenticated_url)

    def _request_login_view(self, data, query=None, follow=False):
        default_data = {}

        default_data.update(data)

        return self.post(
            follow=follow, data=default_data, query=query,
            viewname=settings.LOGIN_URL
        )

    @override_settings(AUTHENTICATION_BACKEND=PATH_AUTHENTICATION_BACKEND_USERNAME)
    def _request_simple_login_view(self, follow=None, query=None):
        AuthenticationBackend.cls_initialize()

        data = {
            'password': self._test_case_user.cleartext_password,
            'username': self._test_case_user.username
        }

        return self._request_login_view(data=data, follow=follow, query=query)

    def _request_login_view_with_email(self, extra_data=None):
        data = {
            'password': self._test_case_super_user.cleartext_password,
            'username': self._test_case_super_user.email
        }

        if extra_data:
            data.update(extra_data)

        return self._request_login_view(data=data)

    def _request_login_view_with_username(
        self, extra_data=None, follow=None, query=None
    ):
        data = {
            'password': self._test_case_super_user.cleartext_password,
            'username': self._test_case_super_user.username
        }

        if extra_data:
            data.update(extra_data)

        return self._request_login_view(data=data, follow=follow, query=query)

    def _request_multi_factor_authentication_view(
        self, data=None, query=None, follow=False
    ):
        default_data = {
            'multi_factor_authentication_view-current_step': '0'
        }

        default_data.update(
            data or {}
        )

        return self.post(
            data=default_data, follow=follow, query=query,
            viewname='authentication:multi_factor_authentication_view'
        )


class LogoutViewTestMixin:
    def _request_logout_view(self):
        return self.post(viewname='authentication:logout_view')


class PasswordResetViewTestMixin:
    def _request_password_reset_confirm_view(self, new_password, uidb64):
        return self.post(
            viewname='authentication:password_reset_confirm_view',
            kwargs={
                'token': PasswordResetConfirmView.reset_url_token,
                'uidb64': uidb64
            }, data={
                'new_password1': new_password,
                'new_password2': new_password
            }
        )

    def _request_password_reset_get_view(self):
        return self.get(
            viewname='authentication:password_reset_view', data={
                'email': self._test_case_super_user.email
            }
        )

    def _request_password_reset_post_view(self):
        return self.post(
            viewname='authentication:password_reset_view', data={
                'email': self._test_case_super_user.email
            }
        )


class UserImpersonationViewTestMixin:
    def _impersonate_test_user(self):
        session = self.client.session
        session[USER_IMPERSONATE_VARIABLE_ID] = self._test_user.pk
        session.save()

    def _request_test_user_impersonate_end_view(self):
        return self.get(
            follow=True, viewname='authentication:user_impersonate_end'
        )

    def _request_test_user_impersonate_form_start_view(self):
        return self.post(
            follow=True, viewname='authentication:user_impersonate_form_start',
            data={
                'user_to_impersonate': self._test_user.pk
            }
        )

    def _request_test_user_impersonate_start_view(self):
        return self.post(
            follow=True, viewname='authentication:user_impersonate_start',
            kwargs={
                'user_id': self._test_user.pk
            }
        )


class UserPasswordViewTestMixin:
    def _request_test_user_password_set_view(self, password):
        return self.post(
            viewname='authentication:user_set_password', kwargs={
                'user_id': self._test_user.pk
            }, data={
                'new_password1': password, 'new_password2': password
            }
        )

    def _request_test_user_password_set_multiple_view(self, password):
        return self.post(
            viewname='authentication:user_multiple_set_password', data={
                'id_list': self._test_user.pk,
                'new_password1': password,
                'new_password2': password
            }
        )



from mayan.apps.credentials.permissions import permission_credential_use
from mayan.apps.documents.events import (
    event_document_created, event_document_file_created,
    event_document_file_edited, event_document_version_created,
    event_document_version_edited, event_document_version_page_created
)
from mayan.apps.documents.models.document_models import Document
from mayan.apps.documents.tests.base import GenericDocumentViewTestCase
from mayan.apps.sources.events import event_source_created
from mayan.apps.sources.models import Source
from mayan.apps.sources.permissions import (
    permission_sources_create, permission_sources_edit
)
from mayan.apps.sources.tests.mixins.source_view_mixins import SourceViewTestMixin

from .literals import TEST_EMAIL_BASE64_FILENAME
from .mixins import (
    EmailSourceTestMixin, IMAPEmailSourceTestMixin, POP3EmailSourceTestMixin
)


class EmailSourceBackendViewTestCase(
    EmailSourceTestMixin, SourceViewTestMixin, GenericDocumentViewTestCase
):
    _test_source_content = TEST_EMAIL_BASE64_FILENAME
    auto_upload_test_document = False

    def test_email_source_create_view_no_permission(self):
        source_count = Source.objects.count()

        self._clear_events()

        response = self._request_test_source_create_post_view()
        self.assertEqual(response.status_code, 403)

        self.assertEqual(Source.objects.count(), source_count)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_email_source_create_view_with_permission(self):
        self.grant_permission(permission=permission_sources_create)

        source_count = Source.objects.count()

        self._clear_events()

        response = self._request_test_source_create_post_view()
        self.assertEqual(response.status_code, 200)

        self.assertEqual(Source.objects.count(), source_count)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_email_source_create_view_with_credential_access(self):
        self.grant_access(
            obj=self._test_stored_credential,
            permission=permission_credential_use
        )

        source_count = Source.objects.count()

        self._clear_events()

        response = self._request_test_source_create_post_view()
        self.assertEqual(response.status_code, 403)

        self.assertEqual(Source.objects.count(), source_count)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_email_source_create_view_with_permission_and_credential_access(self):
        self.grant_permission(permission=permission_sources_create)

        self.grant_access(
            obj=self._test_stored_credential,
            permission=permission_credential_use
        )

        source_count = Source.objects.count()

        self._clear_events()

        response = self._request_test_source_create_post_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(Source.objects.count(), source_count + 1)

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_source)
        self.assertEqual(events[0].verb, event_source_created.id)

    def test_email_source_test_view_no_permission(self):
        document_count = Document.objects.count()

        self._clear_events()

        response = self._request_test_source_test_post_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(
            Document.objects.count(), document_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_email_source_test_view_with_access(self):
        self._silence_logger(name='mayan.apps.converter')

        self.grant_access(
            obj=self._test_source, permission=permission_sources_edit
        )

        document_count = Document.objects.count()

        self._clear_events()

        response = self._request_test_source_test_post_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            Document.objects.count(), document_count + 1
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 6)

        test_document = Document.objects.first()
        test_document_file = test_document.file_latest
        test_document_version = test_document.version_active
        test_document_version_page = test_document_version.pages.first()

        self.assertEqual(events[0].action_object, self._test_document_type)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, test_document)
        self.assertEqual(events[0].verb, event_document_created.id)

        self.assertEqual(events[1].action_object, test_document)
        self.assertEqual(events[1].actor, self._test_case_user)
        self.assertEqual(events[1].target, test_document_file)
        self.assertEqual(events[1].verb, event_document_file_created.id)

        self.assertEqual(events[2].action_object, test_document)
        self.assertEqual(events[2].actor, self._test_case_user)
        self.assertEqual(events[2].target, test_document_file)
        self.assertEqual(events[2].verb, event_document_file_edited.id)

        self.assertEqual(events[3].action_object, test_document)
        self.assertEqual(events[3].actor, self._test_case_user)
        self.assertEqual(events[3].target, test_document_version)
        self.assertEqual(events[3].verb, event_document_version_created.id)

        self.assertEqual(events[4].action_object, test_document_version)
        self.assertEqual(events[4].actor, self._test_case_user)
        self.assertEqual(events[4].target, test_document_version_page)
        self.assertEqual(
            events[4].verb, event_document_version_page_created.id
        )

        self.assertEqual(events[5].action_object, test_document)
        self.assertEqual(events[5].actor, self._test_case_user)
        self.assertEqual(events[5].target, test_document_version)
        self.assertEqual(events[5].verb, event_document_version_edited.id)


class IMAPEmailSourceBackendViewTestCase(
    IMAPEmailSourceTestMixin, SourceViewTestMixin, GenericDocumentViewTestCase
):
    _test_source_content = TEST_EMAIL_BASE64_FILENAME
    auto_upload_test_document = False

    def test_email_source_create_view_no_permission(self):
        source_count = Source.objects.count()

        self._clear_events()

        response = self._request_test_source_create_post_view()
        self.assertEqual(response.status_code, 403)

        self.assertEqual(Source.objects.count(), source_count)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_email_source_create_view_with_permission(self):
        self.grant_permission(permission=permission_sources_create)

        source_count = Source.objects.count()

        self._clear_events()

        response = self._request_test_source_create_post_view()
        self.assertEqual(response.status_code, 200)

        self.assertEqual(Source.objects.count(), source_count)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_email_source_create_view_with_credential_access(self):
        self.grant_access(
            obj=self._test_stored_credential,
            permission=permission_credential_use
        )

        source_count = Source.objects.count()

        self._clear_events()

        response = self._request_test_source_create_post_view()
        self.assertEqual(response.status_code, 403)

        self.assertEqual(Source.objects.count(), source_count)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_email_source_create_view_with_permission_and_credential_access(self):
        self.grant_permission(permission=permission_sources_create)

        self.grant_access(
            obj=self._test_stored_credential,
            permission=permission_credential_use
        )

        source_count = Source.objects.count()

        self._clear_events()

        response = self._request_test_source_create_post_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(Source.objects.count(), source_count + 1)

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_source)
        self.assertEqual(events[0].verb, event_source_created.id)

    def test_email_source_test_view_no_permission(self):
        document_count = Document.objects.count()

        self._clear_events()

        response = self._request_test_source_test_post_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(
            Document.objects.count(), document_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_email_source_test_view_with_access(self):
        self._silence_logger(name='mayan.apps.converter')

        self.grant_access(
            obj=self._test_source, permission=permission_sources_edit
        )

        document_count = Document.objects.count()

        self._clear_events()

        response = self._request_test_source_test_post_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            Document.objects.count(), document_count + 1
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 6)

        test_document = Document.objects.first()
        test_document_file = test_document.file_latest
        test_document_version = test_document.version_active
        test_document_version_page = test_document_version.pages.first()

        self.assertEqual(events[0].action_object, self._test_document_type)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, test_document)
        self.assertEqual(events[0].verb, event_document_created.id)

        self.assertEqual(events[1].action_object, test_document)
        self.assertEqual(events[1].actor, self._test_case_user)
        self.assertEqual(events[1].target, test_document_file)
        self.assertEqual(events[1].verb, event_document_file_created.id)

        self.assertEqual(events[2].action_object, test_document)
        self.assertEqual(events[2].actor, self._test_case_user)
        self.assertEqual(events[2].target, test_document_file)
        self.assertEqual(events[2].verb, event_document_file_edited.id)

        self.assertEqual(events[3].action_object, test_document)
        self.assertEqual(events[3].actor, self._test_case_user)
        self.assertEqual(events[3].target, test_document_version)
        self.assertEqual(events[3].verb, event_document_version_created.id)

        self.assertEqual(events[4].action_object, test_document_version)
        self.assertEqual(events[4].actor, self._test_case_user)
        self.assertEqual(events[4].target, test_document_version_page)
        self.assertEqual(
            events[4].verb, event_document_version_page_created.id
        )

        self.assertEqual(events[5].action_object, test_document)
        self.assertEqual(events[5].actor, self._test_case_user)
        self.assertEqual(events[5].target, test_document_version)
        self.assertEqual(events[5].verb, event_document_version_edited.id)


class POP3EmailSourceBackendViewTestCase(
    POP3EmailSourceTestMixin, SourceViewTestMixin, GenericDocumentViewTestCase
):
    _test_source_content = TEST_EMAIL_BASE64_FILENAME
    auto_upload_test_document = False

    def test_email_source_create_view_no_permission(self):
        source_count = Source.objects.count()

        self._clear_events()

        response = self._request_test_source_create_post_view()
        self.assertEqual(response.status_code, 403)

        self.assertEqual(Source.objects.count(), source_count)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_email_source_create_view_with_permission(self):
        self.grant_permission(permission=permission_sources_create)

        source_count = Source.objects.count()

        self._clear_events()

        response = self._request_test_source_create_post_view()
        self.assertEqual(response.status_code, 200)

        self.assertEqual(Source.objects.count(), source_count)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_email_source_create_view_with_credential_access(self):
        self.grant_access(
            obj=self._test_stored_credential,
            permission=permission_credential_use
        )

        source_count = Source.objects.count()

        self._clear_events()

        response = self._request_test_source_create_post_view()
        self.assertEqual(response.status_code, 403)

        self.assertEqual(Source.objects.count(), source_count)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_email_source_create_view_with_permission_and_credential_access(self):
        self.grant_permission(permission=permission_sources_create)

        self.grant_access(
            obj=self._test_stored_credential,
            permission=permission_credential_use
        )

        source_count = Source.objects.count()

        self._clear_events()

        response = self._request_test_source_create_post_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(Source.objects.count(), source_count + 1)

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_source)
        self.assertEqual(events[0].verb, event_source_created.id)

    def test_email_source_test_view_no_permission(self):
        document_count = Document.objects.count()

        self._clear_events()

        response = self._request_test_source_test_post_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(
            Document.objects.count(), document_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_email_source_test_view_with_access(self):
        self._silence_logger(name='mayan.apps.converter')

        self.grant_access(
            obj=self._test_source, permission=permission_sources_edit
        )

        document_count = Document.objects.count()

        self._clear_events()

        response = self._request_test_source_test_post_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            Document.objects.count(), document_count + 1
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 6)

        test_document = Document.objects.first()
        test_document_file = test_document.file_latest
        test_document_version = test_document.version_active
        test_document_version_page = test_document_version.pages.first()

        self.assertEqual(events[0].action_object, self._test_document_type)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, test_document)
        self.assertEqual(events[0].verb, event_document_created.id)

        self.assertEqual(events[1].action_object, test_document)
        self.assertEqual(events[1].actor, self._test_case_user)
        self.assertEqual(events[1].target, test_document_file)
        self.assertEqual(events[1].verb, event_document_file_created.id)

        self.assertEqual(events[2].action_object, test_document)
        self.assertEqual(events[2].actor, self._test_case_user)
        self.assertEqual(events[2].target, test_document_file)
        self.assertEqual(events[2].verb, event_document_file_edited.id)

        self.assertEqual(events[3].action_object, test_document)
        self.assertEqual(events[3].actor, self._test_case_user)
        self.assertEqual(events[3].target, test_document_version)
        self.assertEqual(events[3].verb, event_document_version_created.id)

        self.assertEqual(events[4].action_object, test_document_version)
        self.assertEqual(events[4].actor, self._test_case_user)
        self.assertEqual(events[4].target, test_document_version_page)
        self.assertEqual(
            events[4].verb, event_document_version_page_created.id
        )

        self.assertEqual(events[5].action_object, test_document)
        self.assertEqual(events[5].actor, self._test_case_user)
        self.assertEqual(events[5].target, test_document_version)
        self.assertEqual(events[5].verb, event_document_version_edited.id)



from django.utils.translation import gettext_lazy as _

from mayan.apps.permissions.classes import PermissionNamespace

namespace = PermissionNamespace(
    label=_(message='Key management'), name='django_gpg'
)

permission_key_delete = namespace.add_permission(
    label=_(message='Delete keys'), name='key_delete'
)
permission_key_download = namespace.add_permission(
    label=_(message='Download keys'), name='key_download'
)
permission_key_receive = namespace.add_permission(
    label=_(message='Import keys from keyservers'), name='key_receive'
)
permission_key_sign = namespace.add_permission(
    label=_(message='Use keys to sign content'), name='key_sign'
)
permission_key_upload = namespace.add_permission(
    label=_(message='Upload keys'), name='key_upload'
)
permission_key_view = namespace.add_permission(
    label=_(message='View keys'), name='key_view'
)
permission_keyserver_query = namespace.add_permission(
    label=_(message='Query keyservers'), name='keyserver_query'
)



from django.utils.translation import gettext_lazy as _

from mayan.apps.smart_settings.settings import setting_cluster

from .literals import (
    DEFAULT_CONVERTER_ASSET_CACHE_MAXIMUM_SIZE,
    DEFAULT_CONVERTER_ASSET_CACHE_STORAGE_BACKEND,
    DEFAULT_CONVERTER_ASSET_CACHE_STORAGE_BACKEND_ARGUMENTS,
    DEFAULT_CONVERTER_ASSET_STORAGE_BACKEND,
    DEFAULT_CONVERTER_ASSET_STORAGE_BACKEND_ARGUMENTS,
    DEFAULT_CONVERTER_GRAPHICS_BACKEND,
    DEFAULT_CONVERTER_GRAPHICS_BACKEND_ARGUMENTS,
    DEFAULT_CONVERTER_IMAGE_CACHE_TIME,
    DEFAULT_CONVERTER_IMAGE_GENERATION_MAX_RETRIES,
    DEFAULT_CONVERTER_IMAGE_GENERATION_TIMEOUT
)
from .setting_callbacks import callback_update_asset_cache_size
from .setting_migrations import ConvertSettingMigration

setting_namespace = setting_cluster.do_namespace_add(
    label=_(message='Converter'), migration_class=ConvertSettingMigration,
    name='converter', version='0002'
)


setting_asset_cache_maximum_size = setting_namespace.do_setting_add(
    default=DEFAULT_CONVERTER_ASSET_CACHE_MAXIMUM_SIZE,
    global_name='CONVERTER_ASSET_CACHE_MAXIMUM_SIZE',
    help_text=_(
        message='The threshold at which the '
        'CONVERTER_ASSET_CACHE_STORAGE_BACKEND '
        'will start deleting the oldest asset cache files. '
        'Specify the size in bytes.'
    ), post_edit_function=callback_update_asset_cache_size
)
setting_asset_cache_storage_backend = setting_namespace.do_setting_add(
    default=DEFAULT_CONVERTER_ASSET_CACHE_STORAGE_BACKEND,
    global_name='CONVERTER_ASSET_CACHE_STORAGE_BACKEND', help_text=_(
        message='Path to the Storage subclass to use when storing the '
        'cached asset files.'
    )
)
setting_asset_cache_storage_backend_arguments = setting_namespace.do_setting_add(
    default=DEFAULT_CONVERTER_ASSET_CACHE_STORAGE_BACKEND_ARGUMENTS,
    global_name='CONVERTER_ASSET_CACHE_STORAGE_BACKEND_ARGUMENTS',
    help_text=_(
        message='Arguments to pass to the '
        'CONVERTER_ASSET_CACHE_STORAGE_BACKEND.'
    )
)
setting_asset_storage_backend = setting_namespace.do_setting_add(
    default=DEFAULT_CONVERTER_ASSET_STORAGE_BACKEND,
    global_name='CONVERTER_ASSET_STORAGE_BACKEND', help_text=_(
        message='Path to the Storage subclass to use when storing assets.'
    )
)
setting_asset_storage_backend_arguments = setting_namespace.do_setting_add(
    default=DEFAULT_CONVERTER_ASSET_STORAGE_BACKEND_ARGUMENTS,
    global_name='CONVERTER_ASSET_STORAGE_BACKEND_ARGUMENTS', help_text=_(
        message='Arguments to pass to the CONVERTER_ASSET_STORAGE_BACKEND.'
    )
)
setting_graphics_backend = setting_namespace.do_setting_add(
    default=DEFAULT_CONVERTER_GRAPHICS_BACKEND,
    global_name='CONVERTER_GRAPHICS_BACKEND', help_text=_(
        message='Graphics conversion backend to use.'
    )
)
setting_graphics_backend_arguments = setting_namespace.do_setting_add(
    default=DEFAULT_CONVERTER_GRAPHICS_BACKEND_ARGUMENTS,
    global_name='CONVERTER_GRAPHICS_BACKEND_ARGUMENTS', help_text=_(
        message='Configuration options for the graphics conversion backend.'
    )
)
setting_image_cache_time = setting_namespace.do_setting_add(
    default=DEFAULT_CONVERTER_IMAGE_CACHE_TIME,
    global_name='CONVERTER_IMAGE_CACHE_TIME',
    help_text=_(
        message='Time in seconds that the browser should cache the '
        'supplied image.'
    )
)
setting_image_generation_max_retries = setting_namespace.do_setting_add(
    default=DEFAULT_CONVERTER_IMAGE_GENERATION_MAX_RETRIES,
    global_name='CONVERTER_IMAGE_GENERATION_MAX_RETRIES',
    help_text=_(
        message='Maximum number of retries before giving up. A value '
        'of None means the task will retry forever.'
    )
)
setting_image_generation_timeout = setting_namespace.do_setting_add(
    default=DEFAULT_CONVERTER_IMAGE_GENERATION_TIMEOUT,
    global_name='CONVERTER_IMAGE_GENERATION_TIMEOUT',
    help_text=_(
        message='Time in seconds after which the image generation task '
        'will stop running and raise an error.'
    )
)



"""
This module should be called settings.py but is named conf.py to avoid a
clash with the mayan/settings/* module
"""
from django.utils.translation import gettext_lazy as _

from mayan.apps.smart_settings.settings import setting_cluster

setting_namespace = setting_cluster.do_namespace_add(
    name='mayan', label=_(message='Mayan')
)

setting_celery_class = setting_namespace.do_setting_add(
    default='celery.Celery', help_text=_(
        message='The class used to instantiate the main Celery app.'
    ), global_name='MAYAN_CELERY_CLASS'
)



COMMAND_NAME_MIRRORING_MOUNT_CABINET = 'mirroring_mount_cabinet'
COMMAND_NAME_MIRRORING_MOUNT_INDEX = 'mirroring_mount_index'

DEFAULT_MIRRORING_DOCUMENT_CACHE_LOOKUP_TIMEOUT = 10
DEFAULT_MIRRORING_NODE_CACHE_LOOKUP_TIMEOUT = 10

FILE_MODE = DIRECTORY_MODE = 0o555

MAX_FILE_DESCRIPTOR = 65535
MIN_FILE_DESCRIPTOR = 0



from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ('common', '0003_auto_20150614_0723'),
    ]

    operations = [
        migrations.DeleteModel(
            name='AnonymousUserSingleton',
        ),
    ]
